@@ -8,33 +8,25 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#ifdef	VMS
-#include <perror.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* For SEEK_* defines if not found in stdio */
 #endif
 
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
-
 #include "st.h"
 
 /* Private data used by writer */
 struct svxpriv {
-        ULONG nsamples;
+	ULONG nsamples;
 	FILE *ch[4];
 };
 
-void svxwriteheader(P2(ft_t, LONG));
-    
+static void svxwriteheader(P2(ft_t, LONG));
+
 /*======================================================================*/
 /*                         8SVXSTARTREAD                                */
 /*======================================================================*/
 
-void svxstartread(ft)
+int st_svxstartread(ft)
 ft_t ft;
 {
 	struct svxpriv *p = (struct svxpriv *) ft->priv;
@@ -45,19 +37,16 @@ ft_t ft;
 	ULONG totalsize;
 	ULONG chunksize;
 
-	int channels;
-	LONG rate;
+	ULONG channels;
+	unsigned short rate;
 	int i;
-	int littlendian = 1;
-	char *endptr;
 
 	ULONG chan1_pos;
 
-	endptr = (char*) &littlendian;
 	/* 8svx is in big endian format. Swap whats
 	 * read in on little endian machines.
 	 */
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
@@ -66,63 +55,94 @@ ft_t ft;
 	channels = 1;
 
 	/* read FORM chunk */
-	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "FORM", 4) != 0)
-		fail("8SVX: header does not begin with magic word 'FORM'");
-	totalsize = rlong(ft);
-	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "8SVX", 4) != 0)
-		fail("8SVX: 'FORM' chunk does not specify '8SVX' as type");
+	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
+	{
+		st_fail_errno(ft, ST_EHDR, "Header did not begin with magic word 'FORM'");
+		return(ST_EOF);
+	}
+	st_readdw(ft, &totalsize);
+	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "8SVX", 4) != 0)
+	{
+		st_fail_errno(ft, ST_EHDR, "'FORM' chunk does not specify '8SVX' as type");
+		return(ST_EOF);
+	}
 
 	/* read chunks until 'BODY' (or end) */
-	while (fread(buf,1,4,ft->fp) == 4 && strncmp(buf,"BODY",4) != 0) {
+	while (st_reads(ft, buf, 4) == ST_SUCCESS && strncmp(buf,"BODY",4) != 0) {
 		if (strncmp(buf,"VHDR",4) == 0) {
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			if (chunksize != 20)
-				fail ("8SVX: VHDR chunk has bad size");
+			{
+				st_fail_errno(ft, ST_EHDR, "VHDR chunk has bad size");
+				return(ST_EOF);
+			}
 			fseek(ft->fp,12,SEEK_CUR);
-			rate = rshort(ft);
+			st_readw(ft, &rate);
 			fseek(ft->fp,1,SEEK_CUR);
 			fread(buf,1,1,ft->fp);
 			if (buf[0] != 0)
-				fail ("8SVX: unsupported data compression");
+			{
+				st_fail_errno(ft, ST_EFMT, "Unsupported data compression");
+				return(ST_EOF);
+			}
 			fseek(ft->fp,4,SEEK_CUR);
 			continue;
 		}
 
 		if (strncmp(buf,"ANNO",4) == 0) {
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			if (chunksize & 1)
 				chunksize++;
-			chunk_buf = (char *) malloc(chunksize + 1);
+			chunk_buf = (char *) malloc(chunksize + 2);
+			if (chunk_buf == 0)
+			{
+			    st_fail_errno(ft, ST_ENOMEM, "Unable to alloc memory");
+			    return(ST_EOF);
+			}
 			if (fread(chunk_buf,1,(size_t)chunksize,ft->fp) 
 					!= chunksize)
-				fail("8SVX: Unexpected EOF in ANNO header");
+			{
+				st_fail_errno(ft, ST_EHDR, "Couldn't read all of header");
+				return(ST_EOF);
+			}
 			chunk_buf[chunksize] = '\0';
-			report ("%s",chunk_buf);
+			st_report("%s",chunk_buf);
 			free(chunk_buf);
 
 			continue;
 		}
 
 		if (strncmp(buf,"NAME",4) == 0) {
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			if (chunksize & 1)
 				chunksize++;
 			chunk_buf = (char *) malloc(chunksize + 1);
+			if (chunk_buf == 0)
+			{
+			    st_fail_errno(ft, ST_ENOMEM, "Unable to alloc memory");
+			    return(ST_EOF);
+			}
 			if (fread (chunk_buf,1,(size_t)chunksize,ft->fp) 
 					!= chunksize)
-				fail("8SVX: Unexpected EOF in NAME header");
+			{
+				st_fail_errno(ft, ST_EHDR, "Couldn't read all of header");
+				return(ST_EOF);
+			}
 			chunk_buf[chunksize] = '\0';
-			report ("%s",chunk_buf);
+			st_report("%s",chunk_buf);
 			free(chunk_buf);
 
 			continue;
 		}
 
 		if (strncmp(buf,"CHAN",4) == 0) {
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			if (chunksize != 4) 
-				fail("8SVX: Short channel chunk");
-			channels = rlong(ft);
+			{
+				st_fail_errno(ft, ST_EHDR, "Couldn't read all of header");
+				return(ST_EOF);
+			}
+			st_readdw(ft, &channels);
 			channels = (channels & 0x01) + 
 					((channels & 0x02) >> 1) +
 				   	((channels & 0x04) >> 2) + 
@@ -132,7 +152,7 @@ ft_t ft;
 		}
 
 		/* some other kind of chunk */
-		chunksize = rlong(ft);
+		st_readdw(ft, &chunksize);
 		if (chunksize & 1)
 			chunksize++;
 		fseek(ft->fp,chunksize,SEEK_CUR);
@@ -141,15 +161,21 @@ ft_t ft;
 	}
 
 	if (rate == 0)
-		fail ("8SVX: invalid rate");
+	{
+		st_fail_errno(ft, ST_ERATE, "Invalid sample rate");
+		return(ST_EOF);
+	}
 	if (strncmp(buf,"BODY",4) != 0)
-		fail ("8SVX: BODY chunk not found");
-	p->nsamples = rlong(ft);
+	{
+		st_fail_errno(ft, ST_EHDR, "BODY chunk not found");
+		return(ST_EOF);
+	}
+	st_readdw(ft, &(p->nsamples));
 
 	ft->info.channels = channels;
 	ft->info.rate = rate;
-	ft->info.style = SIGN2;
-	ft->info.size = BYTE;
+	ft->info.encoding = ST_ENCODING_SIGN2;
+	ft->info.size = ST_SIZE_BYTE;
 
 	/* open files to channels */
 	p->ch[0] = ft->fp;
@@ -157,25 +183,35 @@ ft_t ft;
 
 	for (i = 1; i < channels; i++) {
 		if ((p->ch[i] = fopen(ft->filename, READBINARY)) == NULL)
-			fail("Can't open channel file '%s': %s",
-				ft->filename, strerror(errno));
+		{
+			st_fail_errno(ft,errno,"Can't open channel file '%s'",
+				ft->filename);
+			return(ST_EOF);
+		}
 
 		/* position channel files */
 		if (fseek(p->ch[i],chan1_pos,SEEK_SET))
-		    fail ("Can't position channel %d: %s",i,strerror(errno));
+		{
+		    st_fail_errno (ft,errno,"Can't position channel %d",i);
+		    return(ST_EOF);
+		}
 		if (fseek(p->ch[i],p->nsamples/channels*i,SEEK_CUR))
-		    fail ("Can't seek channel %d: %s",i,strerror(errno));
+		{
+		    st_fail_errno (ft,errno,"Can't seek channel %d",i);
+		    return(ST_EOF);
+		}
 	}
+	return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         8SVXREAD                                     */
 /*======================================================================*/
-LONG svxread(ft, buf, nsamp) 
+LONG st_svxread(ft, buf, nsamp) 
 ft_t ft;
 LONG *buf, nsamp;
 {
-	ULONG datum;
+	unsigned char datum;
 	int done = 0;
 	int i;
 
@@ -183,7 +219,8 @@ LONG *buf, nsamp;
 
 	while (done < nsamp) {
 		for (i = 0; i < ft->info.channels; i++) {
-			datum = getc(p->ch[i]);
+		    	/* FIXME: don't pass FILE pointers! */
+		    	datum = getc(p->ch[i]);
 			if (feof(p->ch[i]))
 				return done;
 			/* scale signed up to long's range */
@@ -197,7 +234,7 @@ LONG *buf, nsamp;
 /*======================================================================*/
 /*                         8SVXSTOPREAD                                 */
 /*======================================================================*/
-void svxstopread(ft)
+int st_svxstopread(ft)
 ft_t ft;
 {
 	int i;
@@ -208,25 +245,22 @@ ft_t ft;
 	for (i = 1; i < ft->info.channels; i++) {
 		fclose (p->ch[i]);
 	}
+	return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         8SVXSTARTWRITE                               */
 /*======================================================================*/
-void svxstartwrite(ft)
+int st_svxstartwrite(ft)
 ft_t ft;
 {
 	struct svxpriv *p = (struct svxpriv *) ft->priv;
 	int i;
 
-	int littlendian = 1;
-	char *endptr;
-
-	endptr = (char *) &littlendian;
 	/* 8svx is in big endian format.  Swaps wahst
 	 * read in on little endian machines.
 	 */
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
@@ -235,29 +269,32 @@ ft_t ft;
 	p->ch[0] = ft->fp;
 	for (i = 1; i < ft->info.channels; i++) {
 		if ((p->ch[i] = tmpfile()) == NULL)
-			fail("Can't open channel output file: %s",
-				strerror(errno));
+		{
+			st_fail_errno(ft,errno,"Can't open channel output file");
+			return(ST_EOF);
+		}
 	}
 
 	/* write header (channel 0) */
-	ft->info.style = SIGN2;
-	ft->info.size = BYTE;
+	ft->info.encoding = ST_ENCODING_SIGN2;
+	ft->info.size = ST_SIZE_BYTE;
 
 	p->nsamples = 0;
 	svxwriteheader(ft, p->nsamples);
+	return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         8SVXWRITE                                    */
 /*======================================================================*/
 
-void svxwrite(ft, buf, len)
+LONG st_svxwrite(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
 	struct svxpriv *p = (struct svxpriv *) ft->priv;
 
-	LONG datum;
+	unsigned char datum;
 	int done = 0;
 	int i;
 
@@ -266,17 +303,19 @@ LONG *buf, len;
 	while(done < len) {
 		for (i = 0; i < ft->info.channels; i++) {
 			datum = RIGHT(*buf++, 24);
-			putc((int)datum, p->ch[i]);
+			/* FIXME: Needs to pass ft struct and not FILE */
+			putc(datum, p->ch[i]);
 		}
 		done += ft->info.channels;
 	}
+	return (done);
 }
 
 /*======================================================================*/
 /*                         8SVXSTOPWRITE                                */
 /*======================================================================*/
 
-void svxstopwrite(ft)
+int st_svxstopwrite(ft)
 ft_t ft;
 {
 	struct svxpriv *p = (struct svxpriv *) ft->priv;
@@ -289,7 +328,10 @@ ft_t ft;
 	/* close temp files */
 	for (i = 1; i < ft->info.channels; i++) {
 		if (fseek (p->ch[i], 0L, 0))
-			fail ("Can't rewind channel output file %d",i);
+		{
+			st_fail_errno (ft,errno,"Can't rewind channel output file %d",i);
+			return(ST_EOF);
+		}
 		while (!feof(p->ch[i])) {
 			len = fread (svxbuf, 1, 512, p->ch[i]);
 			fwrite (svxbuf, 1, len, p->ch[0]);
@@ -299,19 +341,23 @@ ft_t ft;
 
 	/* add a pad byte if BODY size is odd */
 	if(p->nsamples % 2 != 0)
-		fputc('\0', ft->fp);
+	    st_writeb(ft, '\0');
 
 	/* fixup file sizes in header */
 	if (fseek(ft->fp, 0L, 0) != 0)
-		fail("can't rewind output file to rewrite 8SVX header");
+	{
+		st_fail_errno(ft,errno,"can't rewind output file to rewrite 8SVX header");
+		return(ST_EOF);
+	}
 	svxwriteheader(ft, p->nsamples);
+	return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         8SVXWRITEHEADER                              */
 /*======================================================================*/
 #define SVXHEADERSIZE 100
-void svxwriteheader(ft,nsamples)
+static void svxwriteheader(ft,nsamples)
 ft_t ft;
 LONG nsamples;
 {
@@ -320,29 +366,29 @@ LONG nsamples;
 	/* FORM size must be even */
 	if(formsize % 2 != 0) formsize++;
 
-	fputs ("FORM", ft->fp);
-	wlong(ft, formsize);  /* size of file */
-	fputs("8SVX", ft->fp); /* File type */
+	st_writes(ft, "FORM");
+	st_writedw(ft, formsize);  /* size of file */
+	st_writes(ft, "8SVX"); /* File type */
 
-	fputs ("VHDR", ft->fp);
-	wlong(ft, (LONG) 20); /* number of bytes to follow */
-	wlong(ft, nsamples);  /* samples, 1-shot */
-	wlong(ft, (LONG) 0);  /* samples, repeat */
-	wlong(ft, (LONG) 0);  /* samples per repeat cycle */
-	wshort(ft, (int) ft->info.rate); /* samples per second */
-	fputc(1,ft->fp); /* number of octaves */
-	fputc(0,ft->fp); /* data compression (none) */
-	wshort(ft,1); wshort(ft,0); /* volume */
+	st_writes(ft, "VHDR");
+	st_writedw(ft, (LONG) 20); /* number of bytes to follow */
+	st_writedw(ft, nsamples);  /* samples, 1-shot */
+	st_writedw(ft, (LONG) 0);  /* samples, repeat */
+	st_writedw(ft, (LONG) 0);  /* samples per repeat cycle */
+	st_writew(ft, (int) ft->info.rate); /* samples per second */
+	st_writeb(ft,1); /* number of octabes */
+	st_writeb(ft,0); /* data compression (none) */
+	st_writew(ft,1); st_writew(ft,0); /* volume */
 
-	fputs ("ANNO", ft->fp);
-	wlong(ft, (LONG) 32); /* length of block */
-	fputs ("File created by Sound Exchange  ", ft->fp);
+	st_writes(ft, "ANNO");
+	st_writedw(ft, (LONG) 32); /* length of block */
+	st_writes(ft, "File created by Sound Exchange  ");
 
-	fputs ("CHAN", ft->fp);
-	wlong(ft, (LONG) 4);
-	wlong(ft, (ft->info.channels == 2) ? (LONG) 6 :
+	st_writes(ft, "CHAN");
+	st_writedw(ft, (LONG) 4);
+	st_writedw(ft, (ft->info.channels == 2) ? (LONG) 6 :
 		   (ft->info.channels == 4) ? (LONG) 15 : (LONG) 2);
 
-	fputs ("BODY", ft->fp);
-	wlong(ft, nsamples); /* samples in file */
+	st_writes(ft, "BODY");
+	st_writedw(ft, nsamples); /* samples in file */
 }
