@@ -33,9 +33,15 @@
  *   for endianness was being performed AFTER reading the header instead
  *   of before reading it.
  *
+ * Nov 25, 1999 - internal functions made static
+ *
+ * Jul 12, 2000 - Leigh Smith <leigh@tomandandy.com>
+ *   Replaced ANNO with COMT chunk writing headers and added COMT
+ *   chunk reading
  */
 
 #include <math.h>
+#include <time.h>      /* for time stamping comments */
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -44,10 +50,6 @@
 #include <unistd.h>	/* For SEEK_* defines if not found in stdio */
 #endif
 
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
-
 #include "st.h"
 
 /* Private data used by writer */
@@ -55,90 +57,107 @@ struct aiffpriv {
 	ULONG nsamples;	/* number of 1-channel samples read or written */
 };
 
-double read_ieee_extended();
-LONG rawread(P3(ft_t, LONG *, LONG));
-void aiffwriteheader(P2(ft_t, LONG));
-void rawwrite(P3(ft_t, LONG *, LONG));
-void write_ieee_extended(P2(ft_t, double));
-double ConvertFromIeeeExtended();
-void ConvertToIeeeExtended(P2(double, char *));
-void textChunk(P3(char **text, char *chunkDescription, ft_t ft));
-void reportInstrument(P1(ft_t ft));
+/* forward declarations */
+static double read_ieee_extended(P1(ft_t));
+static int aiffwriteheader(P2(ft_t, LONG));
+static void write_ieee_extended(P2(ft_t, double));
+static double ConvertFromIeeeExtended(P1(unsigned char*));
+static void ConvertToIeeeExtended(P2(double, char *));
+static int textChunk(P3(char **text, char *chunkDescription, ft_t ft));
+static int commentChunk(P3(char **text, char *chunkDescription, ft_t ft));
+static void reportInstrument(P1(ft_t ft));
 
-void aiffstartread(ft) 
+int st_aiffstartread(ft) 
 ft_t ft;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
 	char buf[5];
 	ULONG totalsize;
 	LONG chunksize;
-	int channels = 0;
+	unsigned short channels = 0;
 	ULONG frames;
-	int bits = 0;
+	unsigned short bits = 0;
 	double rate = 0.0;
 	ULONG offset = 0;
 	ULONG blocksize = 0;
-	int littlendian = 1;
-	char *endptr;
 	int foundcomm = 0, foundmark = 0, foundinstr = 0;
 	struct mark {
-		int id, position;
+		unsigned short id;
+		ULONG position;
 		char name[40]; 
 	} marks[32];
+	unsigned short looptype;
 	int i, j;
-	LONG nmarks = 0;
-	LONG sustainLoopBegin = 0, sustainLoopEnd = 0,
-	     releaseLoopBegin = 0, releaseLoopEnd = 0;
+	unsigned short nmarks = 0;
+	unsigned short sustainLoopBegin = 0, sustainLoopEnd = 0,
+	     	       releaseLoopBegin = 0, releaseLoopEnd = 0;
 	LONG seekto = 0L, ssndsize = 0L;
 	char *author;
 	char *copyright;
 	char *nametext;
 
-	/* Needed because of rawread() */
-	rawstartread(ft);
+	ULONG trash;
+
+	int rc;
+
+	/* Needed because of st_rawread() */
+	rc = st_rawstartread(ft);
+	if (rc)
+	    return rc;
 
 	/* AIFF is in Big Endian format.  Swap whats read in on Little */
 	/* Endian machines.                                            */
-	endptr = (char *) &littlendian;
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 	    ft->swap = ft->swap ? 0 : 1;
 	}
 
 	/* FORM chunk */
-	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "FORM", 4) != 0)
-		fail("AIFF header does not begin with magic word 'FORM'");
-	totalsize = rlong(ft);
-	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "AIFF", 4) != 0)
-		fail("AIFF 'FORM' chunk does not specify 'AIFF' as type");
+	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
+	{
+		st_fail("AIFF header does not begin with magic word 'FORM'");
+		return(ST_EOF);
+	}
+	st_readdw(ft, &totalsize);
+	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "AIFF", 4) != 0)
+	{
+		st_fail("AIFF 'FORM' chunk does not specify 'AIFF' as type");
+		return(ST_EOF);
+	}
 
 	
 	/* Skip everything but the COMM chunk and the SSND chunk */
 	/* The SSND chunk must be the last in the file */
 	while (1) {
-		if (fread(buf, 1, 4, ft->fp) != 4)
+		if (st_reads(ft, buf, 4) == ST_EOF)
 		{
 			if (ssndsize > 0)
 				break;
 			else
-				fail("Missing SSND chunk in AIFF file");
+			{
+				st_fail("Missing SSND chunk in AIFF file");
+				return(ST_EOF);
+			}
 		}
 		if (strncmp(buf, "COMM", 4) == 0) {
 			/* COMM chunk */
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			if (chunksize != 18)
-				fail("AIFF COMM chunk has bad size");
-			channels = rshort(ft);
-			frames = rlong(ft);
-			bits = rshort(ft);
+			{
+				st_fail("AIFF COMM chunk has bad size");
+				return(ST_EOF);
+			}
+			st_readw(ft, &channels);
+			st_readdw(ft, &frames);
+			st_readw(ft, &bits);
 			rate = read_ieee_extended(ft);
 			foundcomm = 1;
 		}
 		else if (strncmp(buf, "SSND", 4) == 0) {
 			/* SSND chunk */
-			chunksize = rlong(ft);
-			offset = rlong(ft);
-			blocksize = rlong(ft);
+			st_readdw(ft, &chunksize);
+			st_readdw(ft, &offset);
+			st_readdw(ft, &blocksize);
 			chunksize -= 8;
 			ssndsize = chunksize;
 			/* if can't seek, just do sound now */
@@ -150,94 +169,127 @@ ft_t ft;
 		}
 		else if (strncmp(buf, "MARK", 4) == 0) {
 			/* MARK chunk */
-			chunksize = rlong(ft);
-			nmarks = rshort(ft);
+			st_readdw(ft, &chunksize);
+			st_readw(ft, &nmarks);
+
+			/* Some programs like to always have a MARK chunk
+			 * but will set number of marks to 0 and force
+			 * software to detect and ignore it.
+			 */
+			if (nmarks == 0)
+			    foundmark = 0;
+			else
+			    foundmark = 1;
+
 			chunksize -= 2;
 			for(i = 0; i < nmarks; i++) {
-				int len;
+				unsigned char len;
 
-				marks[i].id = rshort(ft);
-				marks[i].position = rlong(ft);
+				st_readw(ft, &(marks[i].id));
+				st_readdw(ft, &(marks[i].position));
 				chunksize -= 6;
-				len = getc(ft->fp);
+				st_readb(ft, &len);
 				chunksize -= len + 1;
 				for(j = 0; j < len ; j++) 
-					marks[i].name[j] = getc(ft->fp);
+				    st_readb(ft, &(marks[i].name[j]));
 				marks[i].name[j] = 0;
 				if ((len & 1) == 0) {
 					chunksize--;
-					getc(ft->fp);
+					st_readb(ft, (unsigned char *)&trash);
 				}
 			}
 			/* HA HA!  Sound Designer (and others) makes */
 			/* bogus files. It spits out bogus chunksize */
 			/* for MARK field */
 			while(chunksize-- > 0)
-				getc(ft->fp);
-			foundmark = 1;
+			    st_readb(ft, (unsigned char *)&trash);
 		}
 		else if (strncmp(buf, "INST", 4) == 0) {
 			/* INST chunk */
-			chunksize = rlong(ft);
-			ft->instr.MIDInote = getc(ft->fp);
-			getc(ft->fp);				/* detune */
-			ft->instr.MIDIlow = getc(ft->fp);
-			ft->instr.MIDIhi = getc(ft->fp);
-			getc(ft->fp);			/* low velocity */
-			getc(ft->fp);			/* hi  velocity */
-			rshort(ft);				/* gain */
-			ft->loops[0].type = rshort(ft); /* sustain loop */
-			sustainLoopBegin = rshort(ft);	 /* begin marker */
-			sustainLoopEnd = rshort(ft);    /* end marker */
-			ft->loops[1].type = rshort(ft); /* release loop */
-			releaseLoopBegin = rshort(ft);  /* begin marker */
-			releaseLoopEnd = rshort(ft);    /* end marker */
+			st_readdw(ft, &chunksize);
+			st_readb(ft, &(ft->instr.MIDInote));
+			st_readb(ft, (unsigned char *)&trash);
+			st_readb(ft, &(ft->instr.MIDIlow));
+			st_readb(ft, &(ft->instr.MIDIhi));
+			/* Low  velocity */
+			st_readb(ft, (unsigned char *)&trash);
+			/* Hi  velocity */
+			st_readb(ft, (unsigned char *)&trash);
+			st_readw(ft, (unsigned short *)&trash);	/* gain */
+			st_readw(ft, &looptype); /* sustain loop */
+			ft->loops[0].type = looptype;
+			st_readw(ft, &sustainLoopBegin); /* begin marker */
+			st_readw(ft, &sustainLoopEnd);    /* end marker */
+			st_readw(ft, &looptype); /* release loop */
+			ft->loops[1].type = looptype;
+			st_readw(ft, &releaseLoopBegin);  /* begin marker */
+			st_readw(ft, &releaseLoopEnd);    /* end marker */
 
 			/* At least one known program generates an INST */
 			/* block with everything zeroed out (meaning    */
 			/* no Loops used).  In this case it should just */
-			/* be ingorned.				        */
+			/* be ignored.				        */
 			if (sustainLoopBegin == 0 && releaseLoopBegin == 0)
 				foundinstr = 0;
 			else
 				foundinstr = 1;
 		}
 		else if (strncmp(buf, "APPL", 4) == 0) {
-			chunksize = rlong(ft);
+			st_readdw(ft, &chunksize);
 			while(chunksize-- > 0)
-				getc(ft->fp);
+			    st_readb(ft, (unsigned char *)&trash);
 		}
 		else if (strncmp(buf, "ALCH", 4) == 0) {
 			/* I think this is bogus and gets grabbed by APPL */
 			/* INST chunk */
-			rlong(ft);		/* ENVS - jeez! */
-			chunksize = rlong(ft);
+			st_readdw(ft, &trash);		/* ENVS - jeez! */
+			st_readdw(ft, &chunksize);
 			while(chunksize-- > 0)
-				getc(ft->fp);
+			    st_readb(ft, (unsigned char *)&trash);
 		}
 		else if (strncmp(buf, "ANNO", 4) == 0) {
-			/* Old form of comment chunk */
-			chunksize = rlong(ft);
-			/* allocate enough memory to hold the comment */
-			ft->comment = (char *) malloc((size_t) chunksize);
-			if (ft->comment == NULL)
-			  fail("AIFF: Couldn't allocate ANNO header");
-			if (fread(ft->comment, 1, chunksize, ft->fp) != chunksize)
-			  fail("AIFF: Unexpected EOF in ANNO header");
+			rc = textChunk(&(ft->comment), "Annotation:", ft);
+			if (rc)
+			{
+			  /* Fail already called in function */
+			  return(ST_EOF);
+			}
+		}
+		else if (strncmp(buf, "COMT", 4) == 0) {
+		  rc = commentChunk(&(ft->comment), "Comment:", ft);
+		  if (rc) {
+		    /* Fail already called in function */
+		    return(ST_EOF);
+		  }
 		}
 		else if (strncmp(buf, "AUTH", 4) == 0) {
 		  /* Author chunk */
-		  textChunk(&author, "Author:", ft);
+		  rc = textChunk(&author, "Author:", ft);
+		  if (rc)
+		  {
+		      /* Fail already called in function */
+		      return(ST_EOF);
+		  }
 		  free(author);
 		}
 		else if (strncmp(buf, "NAME", 4) == 0) {
 		  /* Name chunk */
-		  textChunk(&nametext, "Name:", ft);
+		  rc = textChunk(&nametext, "Name:", ft);
+		  if (rc)
+		  {
+		      /* Fail already called in function */
+		      return(ST_EOF);
+		  }
 		  free(nametext);
 		}
 		else if (strncmp(buf, "(c) ", 4) == 0) {
 		  /* Copyright chunk */
-		  textChunk(&copyright, "Copyright:", ft);
+		  rc = textChunk(&copyright, "Copyright:", ft);
+		  if (rc)
+		  {
+		      /* Fail already called in function */
+		      return(ST_EOF);
+		  }
 		  free(copyright);
 		}
 		else {
@@ -250,14 +302,14 @@ ft_t ft;
 				break;
 			if (feof(ft->fp))
 				break;
-			report("AIFFstartread: ignoring '%s' chunk\n", buf);
-			chunksize = rlong(ft);
+			st_report("AIFFstartread: ignoring '%s' chunk\n", buf);
+			st_readdw(ft, &chunksize);
 			if (feof(ft->fp))
 				break;
-			/* Skip the chunk using getc() so we may read
+			/* Skip the chunk using st_readb() so we may read
 			   from a pipe */
 			while (chunksize-- > 0) {
-				if (getc(ft->fp) == EOF)
+			    if (st_readb(ft, (unsigned char *)&trash) == ST_EOF)
 					break;
 			}
 		}
@@ -274,39 +326,50 @@ ft_t ft;
 		if (seekto > 0)
 			fseek(ft->fp, seekto, SEEK_SET);
 		else
-			fail("AIFF: no sound data on input file");
+		{
+			st_fail("AIFF: no sound data on input file");
+			return(ST_EOF);
+		}
 	}
 	/* SSND chunk just read */
 	if (blocksize != 0)
-		fail("AIFF header specifies nonzero blocksize?!?!");
+	{
+		st_fail("AIFF header specifies nonzero blocksize?!?!");
+		return(ST_EOF);
+	}
 	while ((LONG) (--offset) >= 0) {
-		if (getc(ft->fp) == EOF)
-			fail("unexpected EOF while skipping AIFF offset");
+		if (st_readb(ft, (unsigned char *)&trash) == ST_EOF)
+		{
+			st_fail("unexpected EOF while skipping AIFF offset");
+			return(ST_EOF);
+		}
 	}
 
 	if (foundcomm) {
 		ft->info.channels = channels;
 		ft->info.rate = rate;
-		ft->info.style = SIGN2;
+		ft->info.encoding = ST_ENCODING_SIGN2;
 		switch (bits) {
 		case 8:
-			ft->info.size = BYTE;
+			ft->info.size = ST_SIZE_BYTE;
 			break;
 		case 16:
-			ft->info.size = WORD;
+			ft->info.size = ST_SIZE_WORD;
 			break;
 		default:
-			fail("unsupported sample size in AIFF header: %d", bits);
+			st_fail("unsupported sample size in AIFF header: %d", bits);
+			return(ST_EOF);
 			/*NOTREACHED*/
 		}
 	} else  {
 		if ((ft->info.channels == -1)
 			|| (ft->info.rate == -1)
-			|| (ft->info.style == -1)
+			|| (ft->info.encoding == -1)
 			|| (ft->info.size == -1)) {
-		  report("You must specify # channels, sample rate, signed/unsigned,\n");
-		  report("and 8/16 on the command line.");
-		  fail("Bogus AIFF file: no COMM section.");
+		  st_report("You must specify # channels, sample rate, signed/unsigned,\n");
+		  st_report("and 8/16 on the command line.");
+		  st_fail("Bogus AIFF file: no COMM section.");
+		  return(ST_EOF);
 		}
 
 	}
@@ -314,9 +377,15 @@ ft_t ft;
 	p->nsamples = ssndsize / ft->info.size;	/* leave out channels */
 
 	if (foundmark && !foundinstr)
-		fail("Bogus AIFF file: MARKers but no INSTrument.");
+	{
+		st_fail("Bogus AIFF file: MARKers but no INSTrument.");
+		return(ST_EOF);
+	}
 	if (!foundmark && foundinstr)
-		fail("Bogus AIFF file: INSTrument but no MARKers.");
+	{
+		st_fail("Bogus AIFF file: INSTrument but no MARKers.");
+		return(ST_EOF);
+	}
 	if (foundmark && foundinstr) {
 		int i;
 		int slbIndex = 0, sleIndex = 0;
@@ -341,7 +410,7 @@ ft_t ft;
 			    marks[sleIndex].position - marks[slbIndex].position;
 			/* really the loop count should be infinite */
 			ft->loops[0].count = 1;	
-			ft->instr.loopmode = LOOP_SUSTAIN_DECAY | ft->loops[0].type;
+			ft->instr.loopmode = ST_LOOP_SUSTAIN_DECAY | ft->loops[0].type;
 			ft->instr.nloops++;
 		}
 		if (ft->loops[1].type != 0) {
@@ -350,16 +419,18 @@ ft_t ft;
 			    marks[rleIndex].position - marks[rlbIndex].position;
 			/* really the loop count should be infinite */
 			ft->loops[1].count = 1;
-			ft->instr.loopmode = LOOP_SUSTAIN_DECAY | ft->loops[1].type;
+			ft->instr.loopmode = ST_LOOP_SUSTAIN_DECAY | ft->loops[1].type;
 			ft->instr.nloops++;
 		} 
 	}
 	if (verbose)
 	  reportInstrument(ft);
+
+	return(ST_SUCCESS);
 }
 
 /* print out the MIDI key allocations, loop points, directions etc */
-void reportInstrument(ft)
+static void reportInstrument(ft)
 ft_t ft;
 {
   int loopNum;
@@ -373,7 +444,7 @@ ft_t ft;
 	      ft->loops[loopNum].start + ft->loops[loopNum].length);
       fprintf(stderr, " count: %6d", ft->loops[loopNum].count);
       fprintf(stderr, " type:  ");
-      switch(ft->loops[loopNum].type & ~LOOP_SUSTAIN_DECAY) {
+      switch(ft->loops[loopNum].type & ~ST_LOOP_SUSTAIN_DECAY) {
       case 0: fprintf(stderr, "off\n"); break;
       case 1: fprintf(stderr, "forward\n"); break;
       case 2: fprintf(stderr, "forward/backward\n"); break;
@@ -386,25 +457,99 @@ ft_t ft;
 }
 
 /* Process a text chunk, allocate memory, display it if verbose and return */
-void textChunk(text, chunkDescription, ft) 
+static int textChunk(text, chunkDescription, ft) 
 char **text;
 char *chunkDescription;
 ft_t ft;
 {
-  LONG chunksize = rlong(ft);
+  LONG chunksize;
+  st_readdw(ft, &chunksize);
   /* allocate enough memory to hold the text including a terminating \0 */
   *text = (char *) malloc((size_t) chunksize + 1);
   if (*text == NULL)
-    fail("AIFF: Couldn't allocate %s header", chunkDescription);
+  {
+    st_fail("AIFF: Couldn't allocate %s header", chunkDescription);
+    return(ST_EOF);
+  }
   if (fread(*text, 1, chunksize, ft->fp) != chunksize)
-    fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+  {
+    st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+    return(ST_EOF);
+  }
   *(*text + chunksize) = '\0';
+	if (chunksize % 2)
+	{
+		/* Read past pad byte */
+		char c;
+		if (fread(&c, 1, 1, ft->fp) != 1)
+		{
+			st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+			return(ST_EOF);
+		}
+	}
   if(verbose) {
     printf("%-10s   \"%s\"\n", chunkDescription, *text);
   }
+  return(ST_SUCCESS);
 }
 
-LONG aiffread(ft, buf, len)
+/* Comment lengths are words, not double words, and we can have several, so
+   we use a special function, not textChunk().;
+ */
+static int commentChunk(text, chunkDescription, ft)
+char **text;
+char *chunkDescription;
+ft_t ft;
+{
+  LONG chunksize;
+  unsigned short numComments;
+  LONG timeStamp;
+  unsigned short markerId;
+  unsigned short totalCommentLength = 0;
+  unsigned int commentIndex;
+
+  st_readdw(ft, &chunksize);
+  st_readw(ft, &numComments);
+  for(commentIndex = 0; commentIndex < numComments; commentIndex++) {
+    unsigned short commentLength;
+
+    st_readdw(ft, &timeStamp);
+    st_readw(ft, &markerId);
+    st_readw(ft, &commentLength);
+    totalCommentLength += commentLength;
+    /* allocate enough memory to hold the text including a terminating \0 */
+    if(commentIndex == 0) {
+      *text = (char *) malloc((size_t) totalCommentLength + 1);
+    }
+    else {
+      realloc(*text, (size_t) totalCommentLength + 1);
+    }
+
+    if (*text == NULL) {
+	st_fail("AIFF: Couldn't allocate %s header", chunkDescription);
+	return(ST_EOF);
+    }
+    if (fread(*text + totalCommentLength - commentLength, 1, commentLength, ft->fp) != commentLength) {
+	st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+	return(ST_EOF);
+    }
+    *(*text + totalCommentLength) = '\0';
+    if (commentLength % 2) {
+	/* Read past pad byte */
+	char c;
+	if (fread(&c, 1, 1, ft->fp) != 1) {
+	    st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+	    return(ST_EOF);
+	}
+    }
+  }
+  if(verbose) {
+    printf("%-10s   \"%s\"\n", chunkDescription, *text);
+  }
+  return(ST_SUCCESS);
+}
+
+LONG st_aiffread(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
@@ -413,16 +558,17 @@ LONG *buf, len;
 	/* just read what's left of SSND chunk */
 	if (len > p->nsamples)
 		len = p->nsamples;
-	rawread(ft, buf, len);
+	st_rawread(ft, buf, len);
 	p->nsamples -= len;
 	return len;
 }
 
-void aiffstopread(ft) 
+int st_aiffstopread(ft) 
 ft_t ft;
 {
 	char buf[5];
 	ULONG chunksize;
+	ULONG trash;
 
 	if (!ft->seekable)
 	{
@@ -431,26 +577,24 @@ ft_t ft;
 		if (fread(buf, 1, 4, ft->fp) != 4)
 			break;
 
-		chunksize = rlong(ft);
+		st_readdw(ft, &chunksize);
 		if (feof(ft->fp))
 			break;
 		buf[4] = '\0';
-		warn("Ignoring AIFF tail chunk: '%s', %d bytes long\n", 
+		st_warn("Ignoring AIFF tail chunk: '%s', %d bytes long\n", 
 			buf, chunksize);
 		if (! strcmp(buf, "MARK") || ! strcmp(buf, "INST"))
-			warn("	You're stripping MIDI/loop info!\n");
+			st_warn("	You're stripping MIDI/loop info!\n");
 		while ((LONG) (--chunksize) >= 0) 
 		{
-			if (getc(ft->fp) == EOF)
+			if (st_readb(ft, (unsigned char *)&trash) == ST_EOF)
 				break;
 		}
 	    }
 	}
 
-	/* Needed because of rawwrite() */
-	rawstopread(ft);
-
-	return;
+	/* Needed because of st_rawwrite() */
+	return st_rawstopread(ft);
 }
 
 /* When writing, the header is supposed to contain the number of
@@ -463,31 +607,32 @@ ft_t ft;
    Strictly spoken this is not legal, but the playaiff utility
    will still be able to play the resulting file. */
 
-void aiffstartwrite(ft)
+int st_aiffstartwrite(ft)
 ft_t ft;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
-	int littlendian;
-	char *endptr;
+	int rc;
 
-	/* Needed because rawwrite() */
-	rawstartwrite(ft);
+	/* Needed because st_rawwrite() */
+	rc = st_rawstartwrite(ft);
+	if (rc)
+	    return rc;
 
 	/* AIFF is in Big Endian format.  Swap whats read in on Little */
 	/* Endian machines.                                            */
-	littlendian = 1;
-	endptr = (char *) &littlendian;
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 	    ft->swap = ft->swap ? 0 : 1;
 	}
 
 	p->nsamples = 0;
-	if (ft->info.style == ULAW && ft->info.size == BYTE) {
-		report("expanding 8-bit u-law to 16 bits");
-		ft->info.size = WORD;
+	if ((ft->info.encoding == ST_ENCODING_ULAW ||
+	     ft->info.encoding == ST_ENCODING_ALAW) && 
+	    ft->info.size == ST_SIZE_BYTE) {
+		st_report("expanding 8-bit u-law to 16 bits");
+		ft->info.size = ST_SIZE_WORD;
 	}
-	ft->info.style = SIGN2; /* We have a fixed style */
+	ft->info.encoding = ST_ENCODING_SIGN2; /* We have a fixed encoding */
 
 	/* Compute the "very large number" so that a maximum number
 	   of samples can be transmitted through a pipe without the
@@ -495,37 +640,46 @@ ft_t ft;
 	   At 48 kHz, 16 bits stereo, this gives ~3 hours of music.
 	   Sorry, the AIFF format does not provide for an "infinite"
 	   number of samples. */
-	aiffwriteheader(ft, 0x7f000000L / (ft->info.size*ft->info.channels));
+	return(aiffwriteheader(ft, 0x7f000000L / (ft->info.size*ft->info.channels)));
 }
 
-void aiffwrite(ft, buf, len)
+LONG st_aiffwrite(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
 	p->nsamples += len;
-	rawwrite(ft, buf, len);
+	st_rawwrite(ft, buf, len);
+	return(len);
 }
 
-void
-aiffstopwrite(ft)
+int st_aiffstopwrite(ft)
 ft_t ft;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+	int rc;
 
-	/* Needed because of rawwrite().  Call now to flush
+	/* Needed because of st_rawwrite().  Call now to flush
 	 * buffer now before seeking around below.
 	 */
-	rawstopwrite(ft);
+	rc = st_rawstopwrite(ft);
+	if (rc)
+	    return rc;
 
 	if (!ft->seekable)
-		return;
+	{
+	    st_fail("Non-seekable file.");
+	    return(ST_EOF);
+	}
 	if (fseek(ft->fp, 0L, SEEK_SET) != 0)
-		fail("can't rewind output file to rewrite AIFF header");
-	aiffwriteheader(ft, p->nsamples / ft->info.channels);
+	{
+		st_fail("can't rewind output file to rewrite AIFF header");
+		return(ST_EOF);
+	}
+	return(aiffwriteheader(ft, p->nsamples / ft->info.channels));
 }
 
-void aiffwriteheader(ft, nframes)
+static int aiffwriteheader(ft, nframes)
 ft_t ft;
 LONG nframes;
 {
@@ -534,117 +688,155 @@ LONG nframes;
 		8 /*SSND hdr*/ + 12 /*SSND chunk*/;
 	int bits = 0;
 	int i;
-	int comment_size;
+	int padded_comment_size = 0;
+	int comment_size = 0;
+	LONG comment_chunk_size = 0L;
 
-	hsize += 8 + 2 + 16*ft->instr.nloops;	/* MARK chunk */
-	hsize += 20;				/* INST chunk */
+	/* MARK and INST chunks */
+	if (ft->instr.nloops) {
+	  hsize += 8 /* MARK hdr */ + 2 + 16*ft->instr.nloops;
+	  hsize += 8 /* INST hdr */ + 20; /* INST chunk */
+	}
 
-	if (ft->info.style == SIGN2 && ft->info.size == BYTE)
+	if (ft->info.encoding == ST_ENCODING_SIGN2 && 
+	    ft->info.size == ST_SIZE_BYTE)
 		bits = 8;
-	else if (ft->info.style == SIGN2 && ft->info.size == WORD)
+	else if (ft->info.encoding == ST_ENCODING_SIGN2 && 
+		 ft->info.size == ST_SIZE_WORD)
 		bits = 16;
 	else
-		fail("unsupported output style/size for AIFF header");
+	{
+		st_fail("unsupported output encoding/size for AIFF header");
+		return(ST_EOF);
+	}
 
-	fputs("FORM", ft->fp); /* IFF header */
-	wlong(ft, hsize + nframes * ft->info.size * ft->info.channels); /* file size */
-	fputs("AIFF", ft->fp); /* File type */
-
-	/* ANNO chunk -- holds comments text, however this is */
-	/* discouraged by Apple in preference to a COMT comments */
-	/* chunk, which holds a timestamp and marker id */
+	/* COMT comment chunk -- holds comments text with a timestamp and marker id */
+	/* We calculate the comment_chunk_size if we will be writing a comment */
 	if (ft->comment)
 	{
-	  fputs("ANNO", ft->fp);
-	  /* Must put an even number of characters out.  True 68k processors
-	   * OS's seem to require this 
-	   */
 	  comment_size = strlen(ft->comment);
-	  wlong(ft, (LONG)(((comment_size % 2) == 0) ? comment_size : comment_size + 1)); /* ANNO chunk size, the No of chars */
-	  fputs(ft->comment, ft->fp);
-	  if (comment_size % 2 == 1)
-		fputs(" ", ft->fp);
+	  /* Must put an even number of characters out.
+	   * True 68k processors OS's seem to require this.
+	   */
+	  padded_comment_size = ((comment_size % 2) == 0) ?
+				comment_size : comment_size + 1;
+	  /* one comment, timestamp, marker ID and text count */
+	  comment_chunk_size = (LONG) (2 + 4 + 2 + 2 + padded_comment_size);
+	  hsize += 8 /* COMT hdr */ + comment_chunk_size; 
+	}
+
+	st_writes(ft, "FORM"); /* IFF header */
+	/* file size */
+	st_writedw(ft, hsize + nframes * ft->info.size * ft->info.channels); 
+	st_writes(ft, "AIFF"); /* File type */
+
+	/* Now we write the COMT comment chunk using the precomputed sizes */
+	if (ft->comment)
+	{
+	  st_writes(ft, "COMT");
+	  st_writedw(ft, comment_chunk_size);
+
+	  /* one comment */
+	  st_writew(ft, 1);
+
+	  /* time stamp of comment, Unix knows of time from 1/1/1970,
+	     Apple knows time from 1/1/1904 */
+	  st_writedw(ft, (LONG) ((LONG) time(NULL)) + 2082844800L);
+
+	  /* A marker ID of 0 indicates the comment is not associated
+	     with a marker */
+	  st_writew(ft, 0);
+
+	  /* now write the count and the bytes of text */
+	  st_writew(ft, padded_comment_size);
+	  st_writes(ft, ft->comment);
+	  if (comment_size != padded_comment_size)
+		st_writes(ft, " ");
 	}
 
 	/* COMM chunk -- describes encoding (and #frames) */
-	fputs("COMM", ft->fp);
-	wlong(ft, (LONG) 18); /* COMM chunk size */
-	wshort(ft, ft->info.channels); /* nchannels */
-	wlong(ft, nframes); /* number of frames */
-	wshort(ft, bits); /* sample width, in bits */
+	st_writes(ft, "COMM");
+	st_writedw(ft, (LONG) 18); /* COMM chunk size */
+	st_writew(ft, ft->info.channels); /* nchannels */
+	st_writedw(ft, nframes); /* number of frames */
+	st_writew(ft, bits); /* sample width, in bits */
 	write_ieee_extended(ft, (double)ft->info.rate);
 
 	/* MARK chunk -- set markers */
 	if (ft->instr.nloops) {
-		fputs("MARK", ft->fp);
+		st_writes(ft, "MARK");
 		if (ft->instr.nloops > 2)
 			ft->instr.nloops = 2;
-		wlong(ft, 2 + 16*ft->instr.nloops);
-		wshort(ft, ft->instr.nloops);
+		st_writedw(ft, 2 + 16*ft->instr.nloops);
+		st_writew(ft, ft->instr.nloops);
 
 		for(i = 0; i < ft->instr.nloops; i++) {
-			wshort(ft, i + 1);
-			wlong(ft, ft->loops[i].start);
-			fputc(0, ft->fp);
-			fputc(0, ft->fp);
-			wshort(ft, i*2 + 1);
-			wlong(ft, ft->loops[i].start + ft->loops[i].length);
-			fputc(0, ft->fp);
-			fputc(0, ft->fp);
-			}
+			st_writew(ft, i + 1);
+			st_writedw(ft, ft->loops[i].start);
+			st_writeb(ft, 0);
+			st_writeb(ft, 0);
+			st_writew(ft, i*2 + 1);
+			st_writedw(ft, ft->loops[i].start + ft->loops[i].length);
+			st_writeb(ft, 0);
+			st_writeb(ft, 0);
+		}
 
-		fputs("INST", ft->fp);
-		wlong(ft, 20);
+		st_writes(ft, "INST");
+		st_writedw(ft, 20);
 		/* random MIDI shit that we default on */
-		fputc(ft->instr.MIDInote, ft->fp);
-		fputc(0, ft->fp);			/* detune */
-		fputc(ft->instr.MIDIlow, ft->fp);
-		fputc(ft->instr.MIDIhi, ft->fp);
-		fputc(1, ft->fp);			/* low velocity */
-		fputc(127, ft->fp);			/* hi  velocity */
-		wshort(ft, 0);				/* gain */
+		st_writeb(ft, ft->instr.MIDInote);
+		st_writeb(ft, 0);			/* detune */
+		st_writeb(ft, ft->instr.MIDIlow);
+		st_writeb(ft, ft->instr.MIDIhi);
+		st_writeb(ft, 1);			/* low velocity */
+		st_writeb(ft, 127);			/* hi  velocity */
+		st_writew(ft, 0);				/* gain */
 
 		/* sustain loop */
-		wshort(ft, ft->loops[0].type);
-		wshort(ft, 1);				/* marker 1 */
-		wshort(ft, 3);				/* marker 3 */
+		st_writew(ft, ft->loops[0].type);
+		st_writew(ft, 1);				/* marker 1 */
+		st_writew(ft, 3);				/* marker 3 */
 		/* release loop, if there */
 		if (ft->instr.nloops == 2) {
-			wshort(ft, ft->loops[1].type);
-			wshort(ft, 2);			/* marker 2 */
-			wshort(ft, 4);			/* marker 4 */
+			st_writew(ft, ft->loops[1].type);
+			st_writew(ft, 2);			/* marker 2 */
+			st_writew(ft, 4);			/* marker 4 */
 		} else {
-			wshort(ft, 0);			/* no release loop */
-			wshort(ft, 0);
-			wshort(ft, 0);
+			st_writew(ft, 0);			/* no release loop */
+			st_writew(ft, 0);
+			st_writew(ft, 0);
 		}
 	}
 
 	/* SSND chunk -- describes data */
-	fputs("SSND", ft->fp);
+	st_writes(ft, "SSND");
 	/* chunk size */
-	wlong(ft, 8 + nframes * ft->info.channels * ft->info.size); 
-	wlong(ft, (LONG) 0); /* offset */
-	wlong(ft, (LONG) 0); /* block size */
+	st_writedw(ft, 8 + nframes * ft->info.channels * ft->info.size); 
+	st_writedw(ft, (LONG) 0); /* offset */
+	st_writedw(ft, (LONG) 0); /* block size */
+	return(ST_SUCCESS);
 }
 
-double read_ieee_extended(ft)
+static double read_ieee_extended(ft)
 ft_t ft;
 {
 	char buf[10];
 	if (fread(buf, 1, 10, ft->fp) != 10)
-		fail("EOF while reading IEEE extended number");
+	{
+		st_fail("EOF while reading IEEE extended number");
+		return(ST_EOF);
+	}
 	return ConvertFromIeeeExtended(buf);
 }
 
-void write_ieee_extended(ft, x)
+static void write_ieee_extended(ft, x)
 ft_t ft;
 double x;
 {
 	char buf[10];
 	ConvertToIeeeExtended(x, buf);
 	/*
-	report("converted %g to %o %o %o %o %o %o %o %o %o %o",
+	st_report("converted %g to %o %o %o %o %o %o %o %o %o %o",
 		x,
 		buf[0], buf[1], buf[2], buf[3], buf[4],
 		buf[5], buf[6], buf[7], buf[8], buf[9]);
@@ -694,7 +886,7 @@ double x;
 
 # define FloatToUnsigned(f)      ((ULONG)(((LONG)(f - 2147483648.0)) + 2147483647L) + 1)
 
-void ConvertToIeeeExtended(num, bytes)
+static void ConvertToIeeeExtended(num, bytes)
 double num;
 char *bytes;
 {
@@ -793,7 +985,7 @@ char *bytes;
  * Extended precision IEEE floating-point conversion routine.
  ****************************************************************/
 
-double ConvertFromIeeeExtended(bytes)
+static double ConvertFromIeeeExtended(bytes)
 unsigned char *bytes;	/* LCN */
 {
     double    f;
