@@ -30,6 +30,17 @@
  *        cBW is the band width in Hertz        
  *        cSCL is a scale factor, use 1 for pitched sounds      
  *   use 2 for noise.           
+ *
+ *
+ * July 1, 1999 - Jan Paul Schmidt <jps@fundament.org>
+ *
+ *   This looks like the resonator band pass in SPKit. It's a
+ *   second order all-pole (IIR) band-pass filter described
+ *   at the pages 186 - 189 in
+ *     Dodge, Charles & Jerse, Thomas A. 1985: 
+ *       Computer Music -- Synthesis, Composition and Performance.
+ *       New York: Schirmer Books.  
+ *   Reference from the SPKit manual.
  */
 
 #include <math.h>
@@ -48,7 +59,7 @@ typedef struct bandstuff {
 /*
  * Process options
  */
-void band_getopts(effp, n, argv) 
+int st_band_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
@@ -62,21 +73,31 @@ char **argv;
 		argv++;
 	}
 	if ((n < 1) || !sscanf(argv[0], "%f", &band->center))
-		fail("Usage: band [ -n ] center [ width ]");
+	{
+		st_fail("Usage: band [ -n ] center [ width ]");
+		return (ST_EOF);
+	}
 	band->width = band->center / 2;
 	if ((n >= 2) && !sscanf(argv[1], "%f", &band->width))
-		fail("Usage: band [ -n ] center [ width ]");
+	{
+		st_fail("Usage: band [ -n ] center [ width ]");
+		return (ST_EOF);
+	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare processing.
  */
-void band_start(effp)
+int st_band_start(effp)
 eff_t effp;
 {
 	band_t band = (band_t) effp->priv;
 	if (band->center > effp->ininfo.rate/2)
-		fail("Band: center must be < minimum data rate/2\n");
+	{
+		st_fail("Band: center must be < minimum data rate/2\n");
+		return (ST_EOF);
+	}
 
 	band->C = exp(-2*M_PI*band->width/effp->ininfo.rate);
 	band->B = -4*band->C/(1+band->C)*
@@ -87,6 +108,7 @@ eff_t effp;
 	else
 		band->A = sqrt(1-band->B*band->B/(4*band->C))*(1-band->C);
 	band->out1 = band->out2 = 0.0;
+	return (ST_SUCCESS);
 }
 
 /*
@@ -94,10 +116,10 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void band_flow(effp, ibuf, obuf, isamp, osamp)
+int st_band_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
-int *isamp, *osamp;
+LONG *isamp, *osamp;
 {
 	band_t band = (band_t) effp->priv;
 	int len, done;
@@ -116,15 +138,16 @@ int *isamp, *osamp;
 	}
 	*isamp = len;
 	*osamp = len;
+	return(ST_SUCCESS);
 }
 
 /*
  * Do anything required when you stop reading samples.  
  * Don't close input file! 
  */
-void band_stop(effp)
+int st_band_stop(effp)
 eff_t effp;
 {
-	/* nothing to do */
+	return (ST_SUCCESS);	/* nothing to do */
 }
 
