@@ -63,9 +63,6 @@
  */
 
 #include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
 #include <math.h>
 #include <string.h>
 #include "st.h"
@@ -90,63 +87,10 @@ typedef struct chorusstuff {
 	int	maxsamples, fade_out;
 } *chorus_t;
 
-/* Private data for SKEL file */
-
-LONG chorus_clip24(l)
-LONG l;
-{
-	if (l >= ((LONG)1 << 24))
-		return ((LONG)1 << 24) - 1;
-	else if (l <= -((LONG)1 << 24))
-		return -((LONG)1 << 24) + 1;
-	else
-		return l;
-}
-
-/* This was very painful.  We need a sine library. */
-
-void chorus_sine(buf, len, max, depth)
-int *buf;
-long len;
-int max;
-int depth;
-{
-	long i;
-	int offset;
-	double val;
-
-	offset = max - depth;
-	for (i = 0; i < len; i++) {
-		val = sin((double)i/(double)len * 2.0 * M_PI);
-		buf[i] = offset + (int) (val * (double)depth);
-	}
-}
-
-void chorus_triangle(buf, len, max, depth)
-int *buf;
-long len;
-int max;
-int depth;
-{
-	long i;
-	int offset;
-	double val;
-
-	offset = max - 2 * depth;
-	for (i = 0; i < len / 2; i++) {
-		val = i * 2.0 / len;
-		buf[i] = offset + (int) (val * 2.0 * (double)depth);
-	}
-	for (i = len / 2; i < len ; i++) {
-		val = (len - i) * 2.0 / len;
-		buf[i] = offset + (int) (val * 2.0 * (double)depth);
-	}
-}
-
 /*
  * Process options
  */
-void chorus_getopts(effp, n, argv) 
+int st_chorus_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
@@ -158,13 +102,19 @@ char **argv;
 	i = 0;
 
 	if ( ( n < 7 ) || (( n - 2 ) % 5 ) )
-	    fail("Usage: chorus gain-in gain-out delay decay speed depth [ -s | -t ]");
+	{
+	    st_fail("Usage: chorus gain-in gain-out delay decay speed depth [ -s | -t ]");
+	    return (ST_EOF);
+	}
 
 	sscanf(argv[i++], "%f", &chorus->in_gain);
 	sscanf(argv[i++], "%f", &chorus->out_gain);
 	while ( i < n ) {
 		if ( chorus->num_chorus > MAX_CHORUS )
-			fail("chorus: to many delays, use less than %i delays", MAX_CHORUS);
+		{
+			st_fail("chorus: to many delays, use less than %i delays", MAX_CHORUS);
+			return (ST_EOF);
+		}
 		sscanf(argv[i++], "%f", &chorus->delay[chorus->num_chorus]);
 		sscanf(argv[i++], "%f", &chorus->decay[chorus->num_chorus]);
 		sscanf(argv[i++], "%f", &chorus->speed[chorus->num_chorus]);
@@ -174,16 +124,20 @@ char **argv;
 		else if ( ! strcmp(argv[i], "-t"))
 			chorus->modulation[chorus->num_chorus] = MOD_TRIANGLE;
 		else
-    			fail("Usage: chorus gain-in gain-out delay decay speed [ -s | -t ]");
+		{
+    			st_fail("Usage: chorus gain-in gain-out delay decay speed [ -s | -t ]");
+			return (ST_EOF);
+		}
 		i++;
 		chorus->num_chorus++;
 	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare for processing.
  */
-void chorus_start(effp)
+int st_chorus_start(effp)
 eff_t effp;
 {
 	chorus_t chorus = (chorus_t) effp->priv;
@@ -193,11 +147,20 @@ eff_t effp;
 	chorus->maxsamples = 0;
 
 	if ( chorus->in_gain < 0.0 )
-		fail("chorus: gain-in must be positive!\n");
+	{
+		st_fail("chorus: gain-in must be positive!\n");
+		return (ST_EOF);
+	}
 	if ( chorus->in_gain > 1.0 )
-		fail("chorus: gain-in must be less than 1.0!\n");
+	{
+		st_fail("chorus: gain-in must be less than 1.0!\n");
+		return (ST_EOF);
+	}
 	if ( chorus->out_gain < 0.0 )
-		fail("chorus: gain-out must be positive!\n");
+	{
+		st_fail("chorus: gain-out must be positive!\n");
+		return (ST_EOF);
+	}
 	for ( i = 0; i < chorus->num_chorus; i++ ) {
 		chorus->samples[i] = (int) ( ( chorus->delay[i] + 
 			chorus->depth[i] ) * effp->ininfo.rate / 1000.0);
@@ -205,32 +168,61 @@ eff_t effp;
 			effp->ininfo.rate / 1000.0);
 
 		if ( chorus->delay[i] < 20.0 )
-	    		fail("chorus: delay must be more than 20.0 msec!\n");
+		{
+	    		st_fail("chorus: delay must be more than 20.0 msec!\n");
+			return (ST_EOF);
+		}
 		if ( chorus->delay[i] > 100.0 )
-	    		fail("chorus: delay must be less than 100.0 msec!\n");
+		{
+	    		st_fail("chorus: delay must be less than 100.0 msec!\n");
+			return (ST_EOF);
+		}
 		if ( chorus->speed[i] < 0.1 )
-	    		fail("chorus: speed must be more than 0.1 Hz!\n");
+		{
+	    		st_fail("chorus: speed must be more than 0.1 Hz!\n");
+			return (ST_EOF);
+		}
 		if ( chorus->speed[i] > 5.0 )
-	    		fail("chorus: speed must be less than 5.0 Hz!\n");
+		{
+	    		st_fail("chorus: speed must be less than 5.0 Hz!\n");
+			return (ST_EOF);
+		}
 		if ( chorus->depth[i] < 0.0 )
-	    		fail("chorus: delay must be more positive!\n");
+		{
+	    		st_fail("chorus: delay must be more positive!\n");
+			return (ST_EOF);
+		}
 		if ( chorus->depth[i] > 10.0 )
-	    		fail("chorus: delay must be less than 10.0 msec!\n");
+		{
+		    st_fail("chorus: delay must be less than 10.0 msec!\n");
+		    return (ST_EOF);
+		}
 		if ( chorus->decay[i] < 0.0 )
-	    		fail("chorus: decay must be positive!\n" );
+		{
+	    		st_fail("chorus: decay must be positive!\n" );
+			return (ST_EOF);
+		}
 		if ( chorus->decay[i] > 1.0 )
-	    		fail("chorus: decay must be less that 1.0!\n" );
+		{
+	    		st_fail("chorus: decay must be less that 1.0!\n" );
+			return (ST_EOF);
+		}
 		chorus->length[i] = effp->ininfo.rate / chorus->speed[i];
 		if (! (chorus->lookup_tab[i] = 
 			(int *) malloc(sizeof (int) * chorus->length[i])))
-			fail("chorus: Cannot malloc %d bytes!\n", 
+		{
+			st_fail("chorus: Cannot malloc %d bytes!\n", 
 				sizeof(int) * chorus->length[i]);
+			return (ST_EOF);
+		}
 		if ( chorus->modulation[i] == MOD_SINE )
-			chorus_sine(chorus->lookup_tab[i], chorus->length[i], 
-				chorus->samples[i] - 1, chorus->depth_samples[i]);
+			st_sine(chorus->lookup_tab[i], chorus->length[i], 
+				chorus->depth_samples[i] - 1,
+				chorus->depth_samples[i]);
 		else
-			chorus_triangle(chorus->lookup_tab[i], chorus->length[i], 
-				chorus->samples[i] - 1, chorus->depth_samples[i]);
+			st_triangle(chorus->lookup_tab[i], chorus->length[i], 
+				chorus->samples[i] - 1,
+				chorus->depth_samples[i]);
 		chorus->phase[i] = 0;
 
 		if ( chorus->samples[i] > chorus->maxsamples )
@@ -242,18 +234,22 @@ eff_t effp;
 	for ( i = 0; i < chorus->num_chorus; i++ )
 		sum_in_volume += chorus->decay[i];
 	if ( chorus->in_gain * ( sum_in_volume ) > 1.0 / chorus->out_gain )
-	warn("chorus: warning >>> gain-out can cause saturation or clipping of output <<<");
+	st_warn("chorus: warning >>> gain-out can cause saturation or clipping of output <<<");
 
 
 	if (! (chorus->chorusbuf = 
 		(float *) malloc(sizeof (float) * chorus->maxsamples)))
-		fail("chorus: Cannot malloc %d bytes!\n", 
+	{
+		st_fail("chorus: Cannot malloc %d bytes!\n", 
 			sizeof(float) * chorus->maxsamples);
+		return (ST_EOF);
+	}
 	for ( i = 0; i < chorus->maxsamples; i++ )
 		chorus->chorusbuf[i] = 0.0;
 
 	chorus->counter = 0;
 	chorus->fade_out = chorus->maxsamples;
+	return (ST_SUCCESS);
 }
 
 /*
@@ -261,10 +257,10 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void chorus_flow(effp, ibuf, obuf, isamp, osamp)
+int st_chorus_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
-int *isamp, *osamp;
+LONG *isamp, *osamp;
 {
 	chorus_t chorus = (chorus_t) effp->priv;
 	int len, done;
@@ -285,7 +281,7 @@ int *isamp, *osamp;
 			chorus->maxsamples] * chorus->decay[i];
 		/* Adjust the output volume and size to 24 bit */
 		d_out = d_out * chorus->out_gain;
-		out = chorus_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		*obuf++ = out * 256;
 		/* Mix decay of delay and input */
 		chorus->chorusbuf[chorus->counter] = d_in;
@@ -296,15 +292,16 @@ int *isamp, *osamp;
 				( chorus->phase[i] + 1 ) % chorus->length[i];
 	}
 	/* processed all samples */
+	return (ST_SUCCESS);
 }
 
 /*
  * Drain out reverb lines. 
  */
-void chorus_drain(effp, obuf, osamp)
+int st_chorus_drain(effp, obuf, osamp)
 eff_t effp;
 LONG *obuf;
-int *osamp;
+LONG *osamp;
 {
 	chorus_t chorus = (chorus_t) effp->priv;
 	int done;
@@ -324,7 +321,7 @@ int *osamp;
 		chorus->maxsamples] * chorus->decay[i];
 		/* Adjust the output volume and size to 24 bit */
 		d_out = d_out * chorus->out_gain;
-		out = chorus_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		*obuf++ = out * 256;
 		/* Mix decay of delay and input */
 		chorus->chorusbuf[chorus->counter] = d_in;
@@ -338,12 +335,13 @@ int *osamp;
 	}
 	/* samples played, it remains */
 	*osamp = done;
+	return (ST_SUCCESS);
 }
 
 /*
  * Clean up chorus effect.
  */
-void chorus_stop(effp)
+int st_chorus_stop(effp)
 eff_t effp;
 {
 	chorus_t chorus = (chorus_t) effp->priv;
@@ -355,5 +353,5 @@ eff_t effp;
 		free((char *) chorus->lookup_tab[i]);
 		chorus->lookup_tab[i] = (int *) -1;   /* guaranteed core dump */
 	}
+	return (ST_SUCCESS);
 }
-
