@@ -9,20 +9,19 @@
  *      Thomas Sailer (sailer@ife.ee.ethz.ch) (HB9JNX/AE4WA)
  *      Swiss Federal Institute of Technology, Electronics Lab
  *
- *	This program is free software; you can redistribute it and/or modify
- *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; either version 2 of the License, or
- *	(at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2 of the License, or (at your option) any later version.
  *
- *	This program is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	GNU General Public License for more details.
- *
- *	You should have received a copy of the GNU General Public License
- *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Change History:
  *
@@ -47,7 +46,6 @@
 
 #include "cvsdfilt.h"
 #include "st.h"
-#include "libst.h"
 
 /* ---------------------------------------------------------------------- */
 
@@ -86,7 +84,7 @@ struct cvsdpriv {
 		struct cvsd_encode_state enc;
 	} c;
 	struct {
-		unsigned shreg;
+		unsigned char shreg;
 		unsigned mask;
 		unsigned cnt;
 	} bit;
@@ -97,7 +95,7 @@ struct cvsdpriv {
 
 /* ---------------------------------------------------------------------- */
 
-float float_conv(fp1, fp2, n)
+static float float_conv(fp1, fp2, n)
 float *fp1;
 float *fp2;
 int n;
@@ -125,14 +123,11 @@ ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 	
-	/* sanity check */
-	if (sizeof(struct cvsdpriv) > PRIVSIZE)
-		fail("struct cvsdpriv is too big (%d); change PRIVSIZE in st.h and recompile sox", sizeof(struct cvsdpriv));
 	p->cvsd_rate = (ft->info.rate <= 24000) ? 16000 : 32000;
 	ft->info.rate = 8000;
 	ft->info.channels = 1;
-	ft->info.size = WORD; /* make output format default to words */
-	ft->info.style = SIGN2;
+	ft->info.size = ST_SIZE_WORD; /* make output format default to words */
+	ft->info.encoding = ST_ENCODING_SIGN2;
 	p->swapbits = ft->swap;
 	ft->swap = 0;
 	/*
@@ -160,20 +155,21 @@ ft_t ft;
 	p->bytes_written = 0;
 	p->com.v_min = 1;
 	p->com.v_max = -1;
-	report("cvsd: bit rate %dbit/s, bits from %s\n", p->cvsd_rate,
+	st_report("cvsd: bit rate %dbit/s, bits from %s\n", p->cvsd_rate,
 	       p->swapbits ? "msb to lsb" : "lsb to msb");
 }
 
 /* ---------------------------------------------------------------------- */
 
-void cvsdstartread(ft) 
+int st_cvsdstartread(ft) 
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 	float *fp1;
 	int i;
-	
+
 	cvsdstartcommon(ft);
+
 	p->com.mla_tc1 = 0.1 * (1 - p->com.mla_tc0);
 	p->com.phase = 0;
 	/*
@@ -186,11 +182,13 @@ ft_t ft;
 	 */
 	for(fp1 = p->c.dec.output_filter, i = DEC_FILTERLEN; i > 0; i--)
 		*fp1++ = 0;
+
+	return (ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-void cvsdstartwrite(ft) 
+int st_cvsdstartwrite(ft) 
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
@@ -198,6 +196,7 @@ ft_t ft;
 	int i;
 
 	cvsdstartcommon(ft);
+
 	p->com.mla_tc1 = 0.1 * (1 - p->com.mla_tc0);
 	p->com.phase = 4;
 	/*
@@ -206,34 +205,38 @@ ft_t ft;
 	for(fp1 = p->c.enc.input_filter, i = ENC_FILTERLEN; i > 0; i--)
 		*fp1++ = 0;
 	p->c.enc.recon_int = 0;
+
+	return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-void
-cvsdstopwrite(ft)
+int st_cvsdstopwrite(ft)
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 
 	if (p->bit.cnt) {
-		putc(p->bit.shreg, ft->fp);
+		st_writeb(ft, p->bit.shreg);
 		p->bytes_written++;
 	}
-	report("cvsd: min slope %f, max slope %f\n", 
+	st_report("cvsd: min slope %f, max slope %f\n", 
 	       p->com.v_min, p->com.v_max);	
+
+	return (ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-void
-cvsdstopread(ft)
+int st_cvsdstopread(ft)
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 
-	report("cvsd: min value %f, max value %f\n", 
+	st_report("cvsd: min value %f, max value %f\n", 
 	       p->com.v_min, p->com.v_max);
+
+	return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -248,7 +251,7 @@ static struct {
 } dbg = { NULL, NULL, 0 };
 #endif
 
-LONG cvsdread(ft, buf, nsamp) 
+LONG st_cvsdread(ft, buf, nsamp) 
 ft_t ft;
 LONG *buf, nsamp;
 {
@@ -259,19 +262,24 @@ LONG *buf, nsamp;
 #ifdef DEBUG
 	if (!dbg.f1) {
 		if (!(dbg.f1 = fopen("dbg1", "w")))
-			fail("debugging");
+		{
+			st_fail("debugging");
+			return (0);
+		}
 		fprintf(dbg.f1, "\"input\"\n");
 	}
 	if (!dbg.f2) {
 		if (!(dbg.f2 = fopen("dbg2", "w")))
-			fail("debugging");
+		{
+			st_fail("debugging");
+			return (0);
+		}
 		fprintf(dbg.f2, "\"recon\"\n");
 	}
 #endif
 	while (done < nsamp) {
 		if (!p->bit.cnt) {
-			p->bit.shreg = getc(ft->fp);
-			if (feof(ft->fp))
+		    	if (st_readb(ft, &(p->bit.shreg)) == ST_EOF)
 				return done;
 			p->bit.cnt = 8;
 			p->bit.mask = p->swapbits ? 0x80 : 1;
@@ -325,8 +333,7 @@ LONG *buf, nsamp;
 
 /* ---------------------------------------------------------------------- */
 
-void
-cvsdwrite(ft, buf, nsamp) 
+LONG st_cvsdwrite(ft, buf, nsamp) 
 ft_t ft;
 LONG *buf, nsamp;
 {
@@ -337,12 +344,18 @@ LONG *buf, nsamp;
 #ifdef DEBUG
 	if (!dbg.f1) {
 		if (!(dbg.f1 = fopen("dbg1", "w")))
-			fail("debugging");
+		{
+			st_fail("debugging");
+			return (0);
+		}
 		fprintf(dbg.f1, "\"input\"\n");
 	}
 	if (!dbg.f2) {
 		if (!(dbg.f2 = fopen("dbg2", "w")))
-			fail("debugging");
+		{
+			st_fail("debugging");
+			return (0);
+		}
 		fprintf(dbg.f2, "\"recon\"\n");
 	}
 #endif
@@ -352,7 +365,7 @@ LONG *buf, nsamp;
 		 */
 		if (p->com.phase >= 4) {
 			if (done >= nsamp)
-				return;
+				return done;
 			memmove(p->c.enc.input_filter+1, p->c.enc.input_filter,
 				sizeof(p->c.enc.input_filter)-sizeof(float));
 			p->c.enc.input_filter[0] = (*buf++) / 
@@ -384,7 +397,7 @@ LONG *buf, nsamp;
 		} else
 			p->c.enc.recon_int -= p->com.mla_int;
 		if ((++(p->bit.cnt)) >= 8) {
-			putc(p->bit.shreg, ft->fp);
+		        st_writeb(ft, p->bit.shreg);
 			p->bytes_written++;
 			p->bit.shreg = p->bit.cnt = 0;
 			p->bit.mask = p->swapbits ? 0x80 : 1;
@@ -428,7 +441,7 @@ struct dvms_header {
 #define DVMS_HEADER_LEN 120
 
 /* ---------------------------------------------------------------------- */
-
+/* FIXME: Move these to misc.c */
 static ULONG get32(p)
 unsigned char **p;
 {
@@ -466,7 +479,7 @@ unsigned val;
 
 /* ---------------------------------------------------------------------- */
 
-static void dvms_read_header(f, hdr)
+static int dvms_read_header(f, hdr)
 FILE *f;
 struct dvms_header *hdr;
 {
@@ -476,7 +489,10 @@ struct dvms_header *hdr;
 	unsigned sum;
 
 	if (fread(hdrbuf, sizeof(hdrbuf), 1, f) != 1)
-		fail("unable to read DVMS header\n");
+	{
+		st_fail("unable to read DVMS header\n");
+		return (ST_EOF);
+	}
 	for(i = sizeof(hdrbuf), sum = 0; i > /*2*/3; i--) /* Deti bug */
 		sum += *pch++;
 	pch = hdrbuf;
@@ -498,8 +514,12 @@ struct dvms_header *hdr;
 	pch += sizeof(hdr->extend);
 	hdr->Crc = get16(&pch);
 	if (sum != hdr->Crc) 
-		fail("DVMS header checksum error, read %u, calculated %u\n",
+	{
+		st_fail("DVMS header checksum error, read %u, calculated %u\n",
 		     hdr->Crc, sum);
+		return (ST_EOF);
+	}
+	return (ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -507,7 +527,7 @@ struct dvms_header *hdr;
 /*
  * note! file must be seekable
  */
-static void dvms_write_header(f, hdr)
+static int dvms_write_header(f, hdr)
 FILE *f;
 struct dvms_header *hdr;
 {
@@ -538,9 +558,16 @@ struct dvms_header *hdr;
 	hdr->Crc = sum;
 	put16(&pch, hdr->Crc);
 	if (fseek(f, 0, SEEK_SET) < 0)
-		fail("cannot write DVMS header, seek failed\n");
+	{
+		st_fail("cannot write DVMS header, seek failed\n");
+		return (ST_EOF);
+	}
 	if (fwrite(hdrbuf, sizeof(hdrbuf), 1, f) != 1)
-		fail("cannot write DVMS header\n");
+	{
+		st_fail("cannot write DVMS header\n");
+		return (ST_EOF);
+	}
+	return (ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -573,65 +600,89 @@ struct dvms_header *hdr;
 
 /* ---------------------------------------------------------------------- */
 
-void dvmsstartread(ft) 
+int st_dvmsstartread(ft) 
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 	struct dvms_header hdr;
+	int rc;
 
-	dvms_read_header(ft->fp, &hdr);
-	report("DVMS header of source file \"%s\":");
-	report("  filename  \"%.14s\"",ft->filename);
-        report("  id        0x%x", hdr.Filename);
-	report("  state     0x%x", hdr.Id, hdr.State);
-	report("  time      %s",ctime(&hdr.Unixtime)); /* ctime generates lf */
-	report("  usender   %u", hdr.Usender);
-	report("  ureceiver %u", hdr.Ureceiver);
-	report("  length    %u", hdr.Length);
-	report("  srate     %u", hdr.Srate);
-	report("  days      %u", hdr.Days);
-	report("  custom1   %u", hdr.Custom1);
-	report("  custom2   %u", hdr.Custom2);
-	report("  info      \"%.16s\"\n", hdr.Info);
+	rc = dvms_read_header(ft->fp, &hdr);
+	if (rc)
+	    return rc;
+
+	st_report("DVMS header of source file \"%s\":");
+	st_report("  filename  \"%.14s\"",ft->filename);
+        st_report("  id        0x%x", hdr.Filename);
+	st_report("  state     0x%x", hdr.Id, hdr.State);
+	st_report("  time      %s",ctime(&hdr.Unixtime)); /* ctime generates lf */
+	st_report("  usender   %u", hdr.Usender);
+	st_report("  ureceiver %u", hdr.Ureceiver);
+	st_report("  length    %u", hdr.Length);
+	st_report("  srate     %u", hdr.Srate);
+	st_report("  days      %u", hdr.Days);
+	st_report("  custom1   %u", hdr.Custom1);
+	st_report("  custom2   %u", hdr.Custom2);
+	st_report("  info      \"%.16s\"\n", hdr.Info);
 	ft->info.rate = (hdr.Srate < 240) ? 16000 : 32000;
-	report("DVMS rate %dbit/s using %dbit/s deviation %d%%\n", 
+	st_report("DVMS rate %dbit/s using %dbit/s deviation %d%%\n", 
 	       hdr.Srate*100, ft->info.rate, 
 	       ((ft->info.rate - hdr.Srate*100) * 100) / ft->info.rate);
-	cvsdstartread(ft);
+	rc = st_cvsdstartread(ft);
+	if (rc)
+	    return rc;
+
 	p->swapbits = 0;
+	return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-void dvmsstartwrite(ft) 
+int st_dvmsstartwrite(ft) 
 ft_t ft;
 {
 	struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 	struct dvms_header hdr;
+	int rc;
 	
-	cvsdstartwrite(ft);
+	rc = st_cvsdstartwrite(ft);
+	if (rc)
+	    return rc;
+
 	make_dvms_hdr(ft, &hdr);
-	dvms_write_header(ft->fp, &hdr);
+	rc = dvms_write_header(ft->fp, &hdr);
+	if (rc)
+	    return rc;
+
 	if (!ft->seekable)
-	       warn("Length in output .DVMS header will wrong since can't seek to fix it");
+	       st_warn("Length in output .DVMS header will wrong since can't seek to fix it");
+
 	p->swapbits = 0;
+	return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-void
-dvmsstopwrite(ft)
+int st_dvmsstopwrite(ft)
 ft_t ft;
 {
 	struct dvms_header hdr;
+	int rc;
 	
-	cvsdstopwrite(ft);
+	st_cvsdstopwrite(ft);
 	if (!ft->seekable)
-		return;
+	{
+	    st_warn("File not seekable");
+	    return (ST_EOF);
+	}
 	if (fseek(ft->fp, 0L, 0) != 0)
-		fail("Can't rewind output file to rewrite DVMS header.");
+	{
+		st_fail("Can't rewind output file to rewrite DVMS header.");
+		return(ST_EOF);
+	}
 	make_dvms_hdr(ft, &hdr);
-	dvms_write_header(ft->fp, &hdr);
+	rc = dvms_write_header(ft->fp, &hdr);
+	return rc;
 }
 
 /* ---------------------------------------------------------------------- */
