@@ -56,13 +56,10 @@
  */
 
 #include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
 #include <math.h>
 #include "st.h"
 
-#define DELAY_BUFSIZ ( 50L * MAXRATE )
+#define DELAY_BUFSIZ ( 50L * ST_MAXRATE )
 #define MAX_ECHOS 7	/* 24 bit x ( 1 + MAX_ECHOS ) = */
 			/* 24 bit x 8 = 32 bit !!!	*/
 
@@ -79,24 +76,10 @@ typedef struct echostuff {
 /* Private data for SKEL file */
 
 
-/* If we are not carefull with the output volume */
-LONG echo_clip24(l)
-LONG l;
-{
-	if (l >= ((LONG)1 << 24))
-		return ((LONG)1 << 24) - 1;
-	else if (l <= -((LONG)1 << 24))
-		return -((LONG)1 << 24) + 1;
-	else
-		return l;
-}
-
-
-
 /*
  * Process options
  */
-void echo_getopts(effp, n, argv) 
+int st_echo_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
@@ -107,26 +90,30 @@ char **argv;
 	echo->num_delays = 0;
 
 	if ((n < 4) || (n % 2))
-	    fail("Usage: echo gain-in gain-out delay decay [ delay decay ... ]");
+	{
+	    st_fail("Usage: echo gain-in gain-out delay decay [ delay decay ... ]");
+	    return (ST_EOF);
+	}
 
 	i = 0;
 	sscanf(argv[i++], "%f", &echo->in_gain);
 	sscanf(argv[i++], "%f", &echo->out_gain);
 	while (i < n) {
 		if ( echo->num_delays >= MAX_ECHOS )
-			fail("echo: to many delays, use less than %i delays",
+			st_fail("echo: to many delays, use less than %i delays",
 				MAX_ECHOS);
 		/* Linux bug and it's cleaner. */
 		sscanf(argv[i++], "%f", &echo->delay[echo->num_delays]);
 		sscanf(argv[i++], "%f", &echo->decay[echo->num_delays]);
 		echo->num_delays++;
 	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare for processing.
  */
-void echo_start(effp)
+int st_echo_start(effp)
 eff_t effp;
 {
 	echo_t echo = (echo_t) effp->priv;
@@ -136,28 +123,52 @@ eff_t effp;
 
 	echo->maxsamples = 0L;
 	if ( echo->in_gain < 0.0 )
-		fail("echo: gain-in must be positive!\n");
+	{
+		st_fail("echo: gain-in must be positive!\n");
+		return (ST_EOF);
+	}
 	if ( echo->in_gain > 1.0 )
-		fail("echo: gain-in must be less than 1.0!\n");
+	{
+		st_fail("echo: gain-in must be less than 1.0!\n");
+		return (ST_EOF);
+	}
 	if ( echo->out_gain < 0.0 )
-		fail("echo: gain-in must be positive!\n");
+	{
+		st_fail("echo: gain-in must be positive!\n");
+		return (ST_EOF);
+	}
 	for ( i = 0; i < echo->num_delays; i++ ) {
 		echo->samples[i] = echo->delay[i] * effp->ininfo.rate / 1000.0;
 		if ( echo->samples[i] < 1 )
-		    fail("echo: delay must be positive!\n");
+		{
+		    st_fail("echo: delay must be positive!\n");
+		    return (ST_EOF);
+		}
 		if ( echo->samples[i] > DELAY_BUFSIZ )
-			fail("echo: delay must be less than %g seconds!\n",
+		{
+			st_fail("echo: delay must be less than %g seconds!\n",
 				DELAY_BUFSIZ / (float) effp->ininfo.rate );
+			return (ST_EOF);
+		}
 		if ( echo->decay[i] < 0.0 )
-		    fail("echo: decay must be positive!\n" );
+		{
+		    st_fail("echo: decay must be positive!\n" );
+		    return (ST_EOF);
+		}
 		if ( echo->decay[i] > 1.0 )
-		    fail("echo: decay must be less than 1.0!\n" );
+		{
+		    st_fail("echo: decay must be less than 1.0!\n" );
+		    return (ST_EOF);
+		}
 		if ( echo->samples[i] > echo->maxsamples )
 			echo->maxsamples = echo->samples[i];
 	}
 	if (! (echo->delay_buf = (double *) malloc(sizeof (double) * echo->maxsamples)))
-		fail("echo: Cannot malloc %d bytes!\n", 
+	{
+		st_fail("echo: Cannot malloc %d bytes!\n", 
 			sizeof(long) * echo->maxsamples);
+		return (ST_EOF);
+	}
 	for ( j = 0; j < echo->maxsamples; ++j )
 		echo->delay_buf[j] = 0.0;
 	/* Be nice and check the hint with warning, if... */
@@ -165,9 +176,10 @@ eff_t effp;
 	for ( i = 0; i < echo->num_delays; i++ ) 
 		sum_in_volume += echo->decay[i];
 	if ( sum_in_volume * echo->in_gain > 1.0 / echo->out_gain )
-		warn("echo: warning >>> gain-out can cause saturation of output <<<");
+		st_warn("echo: warning >>> gain-out can cause saturation of output <<<");
 	echo->counter = 0;
 	echo->fade_out = echo->maxsamples;
+	return (ST_SUCCESS);
 }
 
 /*
@@ -175,10 +187,10 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void echo_flow(effp, ibuf, obuf, isamp, osamp)
+int st_echo_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
-int *isamp, *osamp;
+LONG *isamp, *osamp;
 {
 	echo_t echo = (echo_t) effp->priv;
 	int len, done;
@@ -200,7 +212,7 @@ int *isamp, *osamp;
 		}
 		/* Adjust the output volume and size to 24 bit */
 		d_out = d_out * echo->out_gain;
-		out = echo_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		*obuf++ = out * 256;
 		/* Store input in delay buffer */
 		echo->delay_buf[echo->counter] = d_in;
@@ -208,15 +220,16 @@ int *isamp, *osamp;
 		echo->counter = ( echo->counter + 1 ) % echo->maxsamples;
 	}
 	/* processed all samples */
+	return (ST_SUCCESS);
 }
 
 /*
  * Drain out reverb lines. 
  */
-void echo_drain(effp, obuf, osamp)
+int st_echo_drain(effp, obuf, osamp)
 eff_t effp;
 LONG *obuf;
-int *osamp;
+LONG *osamp;
 {
 	echo_t echo = (echo_t) effp->priv;
 	double d_in, d_out;
@@ -236,7 +249,7 @@ int *osamp;
 		}
 		/* Adjust the output volume and size to 24 bit */
 		d_out = d_out * echo->out_gain;
-		out = echo_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		*obuf++ = out * 256;
 		/* Store input in delay buffer */
 		echo->delay_buf[echo->counter] = d_in;
@@ -247,17 +260,18 @@ int *osamp;
 	};
 	/* samples played, it remains */
 	*osamp = done;
+	return (ST_SUCCESS);
 }
 
 /*
  * Clean up reverb effect.
  */
-void echo_stop(effp)
+int st_echo_stop(effp)
 eff_t effp;
 {
 	echo_t echo = (echo_t) effp->priv;
 
 	free((char *) echo->delay_buf);
 	echo->delay_buf = (double *) -1;   /* guaranteed core dump */
+	return (ST_SUCCESS);
 }
-
