@@ -29,7 +29,7 @@
  *
  * Limited to 16 bit samples and sampling frequency <= 65535 Hz. If
  * the input & output frequencies are equal, a delay of one sample is
- * introduced 
+ * introduced.  Limited to processing 32-bit count worth of samples.
  *
  * 1 << FRAC_BITS evaluating to zero in several places.  Changed with
  * an (unsigned long) cast to make it safe.  MarkMLl 2/1/99
@@ -39,13 +39,13 @@
 
 /* Private data */
 typedef struct ratestuff {
-        u_l opos_frac;  /* fractional position of the output stream in input stream unit */
-        u_l opos;
+        ULONG opos_frac;  /* fractional position of the output stream in input stream unit */
+        ULONG opos;
 
-        u_l opos_inc_frac;  /* fractional position increment in the output stream */
-        u_l opos_inc; 
+        ULONG opos_inc_frac;  /* fractional position increment in the output stream */
+        ULONG opos_inc; 
 
-        u_l ipos;      /* position in the input stream (integer) */
+        ULONG ipos;      /* position in the input stream (integer) */
 
         LONG ilast; /* last sample in the input stream */
 } *rate_t;
@@ -53,29 +53,44 @@ typedef struct ratestuff {
 /*
  * Process options
  */
-void rate_getopts(effp, n, argv) 
+int st_rate_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
 {
 	if (n)
-		fail("Rate effect takes no options.");
+	{
+		st_fail("Rate effect takes no options.");
+		return (ST_EOF);
+	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare processing.
  */
-void rate_start(effp)
+int st_rate_start(effp)
 eff_t effp;
 {
 	rate_t rate = (rate_t) effp->priv;
-        u_l incr;
+        ULONG incr;
+
+	if (effp->ininfo.rate >= 65535 || effp->outinfo.rate >= 65535)
+	{
+	    st_fail("rate effect can only handle rates <= 65535");
+	    return (ST_EOF);
+	}
+	if (effp->ininfo.size == ST_SIZE_DWORD ||
+	    effp->ininfo.size == ST_SIZE_FLOAT)
+	{
+	    st_warn("rate effect reduces data to 16 bits");
+	}
 
         rate->opos_frac=0;
         rate->opos=0;
 
         /* increment */
-        incr=(u_l)((double)effp->ininfo.rate / (double)effp->outinfo.rate * 
+        incr=(ULONG)((double)effp->ininfo.rate / (double)effp->outinfo.rate * 
                    (double) ((unsigned long) 1 << FRAC_BITS));
 
         rate->opos_inc_frac = incr & (((unsigned long) 1 << FRAC_BITS)-1);
@@ -84,6 +99,7 @@ eff_t effp;
         rate->ipos=0;
 
 	rate->ilast = 0;
+	return (ST_SUCCESS);
 }
 
 /*
@@ -91,16 +107,16 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void rate_flow(effp, ibuf, obuf, isamp, osamp)
+int st_rate_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
-int *isamp, *osamp;
+LONG *isamp, *osamp;
 {
 	rate_t rate = (rate_t) effp->priv;
 	LONG *istart,*iend;
 	LONG *ostart,*oend;
 	LONG ilast,icur,out;
-        u_l tmp;
+        ULONG tmp;
         double t;
 
         ilast=rate->ilast;
@@ -113,13 +129,18 @@ int *isamp, *osamp;
 
         while (obuf < oend) {
 
+		/* Safety catch to make sure we have input samples.  */
+                if (ibuf >= iend) goto the_end;
+
                 /* read as many input samples so that ipos > opos */
-                
+        
                 while (rate->ipos <= rate->opos) {
-                        if (ibuf >= iend) goto the_end;
                         ilast = *ibuf++;
                         rate->ipos++;
+			/* See if we finished the input buffer yet */
+                        if (ibuf >= iend) goto the_end;
                 }
+
                 icur = *ibuf;
         
                 /* interpolate */
@@ -138,16 +159,18 @@ the_end:
 	*isamp = ibuf - istart;
 	*osamp = obuf - ostart;
 	rate->ilast = ilast;
+	return (ST_SUCCESS);
 }
 
 /*
  * Do anything required when you stop reading samples.  
  * Don't close input file! 
  */
-void rate_stop(effp)
+int st_rate_stop(effp)
 eff_t effp;
 {
 	/* nothing to do */
+    return (ST_SUCCESS);
 }
 
 #else /* USE_OLD_RATE */
@@ -205,33 +228,36 @@ eff_t effp;
 
 /* Private data for Lerp via LCM file */
 typedef struct ratestuff {
-	u_l	lcmrate;		/* least common multiple of rates */
-	u_l	inskip, outskip;	/* LCM increments for I & O rates */
-	u_l	total;
-	u_l	intot, outtot;		/* total samples in LCM basis */
+	ULONG	lcmrate;		/* least common multiple of rates */
+	ULONG	inskip, outskip;	/* LCM increments for I & O rates */
+	ULONG	total;
+	ULONG	intot, outtot;		/* total samples in LCM basis */
 	LONG	lastsamp;		/* history */
 } *rate_t;
 
 /*
  * Process options
  */
-void rate_getopts(effp, n, argv) 
+int st_rate_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
 {
 	if (n)
-		fail("Rate effect takes no options.");
+	{
+		st_fail("Rate effect takes no options.");
+		return (ST_EOF);
+	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare processing.
  */
-void rate_start(effp)
+int st_rate_start(effp)
 eff_t effp;
 {
 	rate_t rate = (rate_t) effp->priv;
-	IMPORT LONG lcm();
 	
 	rate->lcmrate = lcm((LONG)effp->ininfo.rate, (LONG)effp->outinfo.rate);
 	/* Cursory check for LCM overflow.  
@@ -242,6 +268,7 @@ eff_t effp;
 	rate->outskip = rate->lcmrate / effp->outinfo.rate; 
 	rate->total = rate->intot = rate->outtot = 0;
 	rate->lastsamp = 0;
+	return (ST_SUCCESS);
 }
 
 /*
@@ -249,7 +276,7 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void rate_flow(effp, ibuf, obuf, isamp, osamp)
+int st_rate_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
 int *isamp, *osamp;
@@ -305,15 +332,17 @@ out:
 	*isamp = ibuf - istart;
 	*osamp = len;
 	rate->lastsamp = last;
+	return (ST_SUCCESS);
 }
 
 /*
  * Do anything required when you stop reading samples.  
  * Don't close input file! 
  */
-void rate_stop(effp)
+int st_rate_stop(effp)
 eff_t effp;
 {
 	/* nothing to do */
+    return (ST_SUCCESS);
 }
 #endif /* USE_OLD_RATE */
