@@ -93,14 +93,11 @@
  */
 
 #include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
 #include <math.h>
 #include "st.h"
 
 #define REVERB_FADE_THRESH 10
-#define DELAY_BUFSIZ ( 50L * MAXRATE )
+#define DELAY_BUFSIZ ( 50L * ST_MAXRATE )
 #define MAXREVERBS 8
 
 /* Private data for SKEL file */
@@ -114,27 +111,10 @@ typedef struct reverbstuff {
 	LONG	pl, ppl, pppl;
 } *reverb_t;
 
-/* Private data for SKEL file */
-
-#ifndef abs
-#define abs(a) ((a) >= 0 ? (a) : -(a))
-#endif
-
-LONG reverb_clip24(l)
-LONG l;
-{
-	if (l >= ((LONG)1 << 24))
-		return ((LONG)1 << 24) - 1;
-	else if (l <= -((LONG)1 << 24))
-		return -((LONG)1 << 24) + 1;
-	else
-		return l;
-}
-
 /*
  * Process options
  */
-void reverb_getopts(effp, n, argv) 
+int st_reverb_getopts(effp, n, argv) 
 eff_t effp;
 int n;
 char **argv;
@@ -146,11 +126,17 @@ char **argv;
 	reverb->maxsamples = 0;
 
 	if ( n < 3 )
-	    fail("Usage: reverb gain-out reverb-time delay [ delay ... ]");
+	{
+	    st_fail("Usage: reverb gain-out reverb-time delay [ delay ... ]");
+	    return (ST_EOF);
+	}
 
 	if ( n - 2 > MAXREVERBS )
-	    fail("reverb: to many dalays, use less than %i delays",
+	{
+	    st_fail("reverb: to many dalays, use less than %i delays",
 			MAXREVERBS);
+	    return (ST_EOF);
+	}
 
 	i = 0;
 	sscanf(argv[i++], "%f", &reverb->out_gain);
@@ -160,12 +146,13 @@ char **argv;
 		sscanf(argv[i++], "%f", &reverb->delay[reverb->numdelays]);
 		reverb->numdelays++;
 	}
+	return (ST_SUCCESS);
 }
 
 /*
  * Prepare for processing.
  */
-void reverb_start(effp)
+int st_reverb_start(effp)
 eff_t effp;
 {
 	reverb_t reverb = (reverb_t) effp->priv;
@@ -174,26 +161,41 @@ eff_t effp;
 	reverb->in_gain = 1.0;
 
 	if ( reverb->out_gain < 0.0 )
-		fail("reverb: gain-out must be positive");
+	{
+		st_fail("reverb: gain-out must be positive");
+		return (ST_EOF);
+	}
 	if ( reverb->out_gain > 1.0 )
-		warn("reverb: warnig >>> gain-out can cause saturation of output <<<");
+		st_warn("reverb: warnig >>> gain-out can cause saturation of output <<<");
 	if ( reverb->time < 0.0 )
-		fail("reverb: reverb-time must be positive");
+	{
+		st_fail("reverb: reverb-time must be positive");
+		return (ST_EOF);
+	}
 	for(i = 0; i < reverb->numdelays; i++) {
 		reverb->samples[i] = reverb->delay[i] * effp->ininfo.rate / 1000.0;
 		if ( reverb->samples[i] < 1 )
-		    fail("reverb: delay must be positive!\n");
+		{
+		    st_fail("reverb: delay must be positive!\n");
+		    return (ST_EOF);
+		}
 		if ( reverb->samples[i] > DELAY_BUFSIZ )
-			fail("reverb: delay must be less than %g seconds!\n",
+		{
+			st_fail("reverb: delay must be less than %g seconds!\n",
 				DELAY_BUFSIZ / (float) effp->ininfo.rate );
+			return(ST_EOF);
+		}
 		/* Compute a realistic decay */
 		reverb->decay[i] = (float) pow(10.0,(-3.0 * reverb->delay[i] / reverb->time));
 		if ( reverb->samples[i] > reverb->maxsamples )
 		    reverb->maxsamples = reverb->samples[i];
 	}
 	if (! (reverb->reverbbuf = (float *) malloc(sizeof (float) * reverb->maxsamples)))
-		fail("reverb: Cannot malloc %d bytes!\n", 
+	{
+		st_fail("reverb: Cannot malloc %d bytes!\n", 
 			sizeof(float) * reverb->maxsamples);
+		return(ST_EOF);
+	}
 	for ( i = 0; i < reverb->maxsamples; ++i )
 		reverb->reverbbuf[i] = 0.0;
 	reverb->pppl = reverb->ppl = reverb->pl = 0x7fffff;		/* fade-outs */
@@ -202,6 +204,7 @@ eff_t effp;
 	for ( i = 0; i < reverb->numdelays; i++ )
 		reverb->in_gain *= 
 			( 1.0 - ( reverb->decay[i] * reverb->decay[i] ));
+	return (ST_SUCCESS);
 }
 
 /*
@@ -209,10 +212,10 @@ eff_t effp;
  * Return number of samples processed.
  */
 
-void reverb_flow(effp, ibuf, obuf, isamp, osamp)
+int st_reverb_flow(effp, ibuf, obuf, isamp, osamp)
 eff_t effp;
 LONG *ibuf, *obuf;
-int *isamp, *osamp;
+LONG *isamp, *osamp;
 {
 	reverb_t reverb = (reverb_t) effp->priv;
 	int len, done;
@@ -232,7 +235,7 @@ int *isamp, *osamp;
 			d_in +=
 reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsamples] * reverb->decay[j];
 		d_out = d_in * reverb->out_gain;
-		out = reverb_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		*obuf++ = out * 256;
 		reverb->reverbbuf[i] = d_in;
 		i++;		/* XXX need a % maxsamples here ? */
@@ -240,15 +243,16 @@ reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsam
 	}
 	reverb->counter = i;
 	/* processed all samples */
+	return (ST_SUCCESS);
 }
 
 /*
  * Drain out reverb lines. 
  */
-void reverb_drain(effp, obuf, osamp)
+int st_reverb_drain(effp, obuf, osamp)
 eff_t effp;
 LONG *obuf;
-int *osamp;
+LONG *osamp;
 {
 	reverb_t reverb = (reverb_t) effp->priv;
 	float d_in, d_out;
@@ -265,10 +269,10 @@ int *osamp;
 			d_in += 
 reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsamples] * reverb->decay[j];
 		d_out = d_in * reverb->out_gain;
-		out = reverb_clip24((LONG) d_out);
+		out = st_clip24((LONG) d_out);
 		obuf[done++] = out * 256;
 		reverb->reverbbuf[i] = d_in;
-		l = reverb_clip24((LONG) d_in);
+		l = st_clip24((LONG) d_in);
 		reverb->pppl = reverb->ppl;
 		reverb->ppl = reverb->pl;
 		reverb->pl = l;
@@ -278,17 +282,18 @@ reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsam
 		((abs(reverb->pl) + abs(reverb->ppl) + abs(reverb->pppl)) > REVERB_FADE_THRESH));
 	reverb->counter = i;
 	*osamp = done;
+	return (ST_SUCCESS);
 }
 
 /*
  * Clean up reverb effect.
  */
-void reverb_stop(effp)
+int st_reverb_stop(effp)
 eff_t effp;
 {
 	reverb_t reverb = (reverb_t) effp->priv;
 
 	free((char *) reverb->reverbbuf);
 	reverb->reverbbuf = (float *) -1;   /* guaranteed core dump */
+	return (ST_SUCCESS);
 }
-
