@@ -26,116 +26,130 @@ struct sndpriv {
 #define	SEEK_CUR	1
 #endif
 
-void  sndtwriteheader(P2(ft_t ft,LONG nsamples));
-void rawwrite(P3(ft_t, LONG *, LONG));
+static void  sndtwriteheader(P2(ft_t ft,LONG nsamples));
 
 /*======================================================================*/
 /*                         SNDSTARTREAD                                */
 /*======================================================================*/
 
-void sndtstartread(ft)
+int st_sndtstartread(ft)
 ft_t ft;
 {
-char buf[97];
+        char buf[97];
 
-LONG rate;
-
-	int littlendian = 1;
-	char *endptr;
+        unsigned short rate;
+	int rc;
 
 	/* Needed for rawread() */
-	rawstartread(ft);
+	rc = st_rawstartread(ft);
+	if (rc)
+	    return rc;
 
-	endptr = (char *) &littlendian;
 	/* sndt is in little endian format so 
 	 * swap bytes on big endian machines.
 	 */
-	if (!*endptr)
+	if (ST_IS_BIGENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
 
 	rate = 0;
 
-/* determine file type */
+	/* determine file type */
         /* if first 5 bytes == SOUND then this is probably a sndtool sound */
         /* if first word (16 bits) == 0 
          and second word is between 4000 & 25000 then this is sounder sound */
         /* otherwise, its probably raw, not handled here */
 
-if (fread(buf, 1, 2, ft->fp) != 2)
-	fail("SND: unexpected EOF");
-if (strncmp(buf,"\0\0",2) == 0)
+	if (fread(buf, 1, 2, ft->fp) != 2)
+	{
+		st_fail("SND: unexpected EOF");
+		return(ST_EOF);
+	}
+	if (strncmp(buf,"\0\0",2) == 0)
 	{
 	/* sounder */
-	rate = rshort(ft);
+	st_readw(ft, &rate);
 	if (rate < 4000 || rate > 25000 )
-		fail ("SND: sample rate out of range");
+	{
+		st_fail("SND: sample rate out of range");
+		return(ST_EOF);
+	}
 	fseek(ft->fp,4,SEEK_CUR);
 	}
-else
+	else
 	{
 	/* sndtool ? */
-	fread(&buf[2],1,6,ft->fp);
+	fread(&buf[2], 1, 6, ft->fp);
 	if (strncmp(buf,"SOUND",5))
-		fail ("SND: unrecognized SND format");
+	{
+		st_fail("SND: unrecognized SND format");
+		return(ST_EOF);
+	}
 	fseek(ft->fp,12,SEEK_CUR);
-	rate = rshort(ft);
+	st_readw(ft, &rate);
 	fseek(ft->fp,6,SEEK_CUR);
-	if (fread(buf,1,96,ft->fp) != 96)
-		fail ("SND: unexpected EOF in SND header");
-	report ("%s",buf);
+	if (st_reads(ft, buf, 96) == ST_EOF)
+	{
+		st_fail("SND: unexpected EOF in SND header");
+		return(ST_EOF);
+	}
+	st_report("%s",buf);
 	}
 
 ft->info.channels = 1;
 ft->info.rate = rate;
-ft->info.style = UNSIGNED;
-ft->info.size = BYTE;
+ft->info.encoding = ST_ENCODING_UNSIGNED;
+ft->info.size = ST_SIZE_BYTE;
 
+return (ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         SNDTSTARTWRITE                               */
 /*======================================================================*/
-void sndtstartwrite(ft)
+int st_sndtstartwrite(ft)
 ft_t ft;
 {
-struct sndpriv *p = (struct sndpriv *) ft->priv;
-
-	int littlendian = 1;
-	char *endptr;
+	struct sndpriv *p = (struct sndpriv *) ft->priv;
+	int rc;
 
 	/* Needed for rawwrite() */
-	rawstartwrite(ft);
+	rc = st_rawstartwrite(ft);
+	if (rc)
+	    return rc;
 
-	endptr = (char *) &littlendian;
 	/* sndt is in little endian format so
 	 * swap bytes on big endian machines
 	 */
-	if (!*endptr)
+	if (ST_IS_BIGENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
 
 /* write header */
 ft->info.channels = 1;
-ft->info.style = UNSIGNED;
-ft->info.size = BYTE;
+ft->info.encoding = ST_ENCODING_UNSIGNED;
+ft->info.size = ST_SIZE_BYTE;
 p->nsamples = 0;
 sndtwriteheader(ft, 0);
 
+return(ST_SUCCESS);
 }
 /*======================================================================*/
 /*                         SNDRSTARTWRITE                               */
 /*======================================================================*/
-void sndrstartwrite(ft)
+int st_sndrstartwrite(ft)
 ft_t ft;
 {
 	int littlendian = 1;
 	char *endptr;
+	int rc;
 
 	/* Needed for rawread() */
-	rawstartread(ft);
+	rc = st_rawstartread(ft);
+	if (rc)
+	    return rc;
 
 	endptr = (char *) &littlendian;
 	/* sndr is in little endian format so
@@ -148,70 +162,76 @@ ft_t ft;
 
 /* write header */
 ft->info.channels = 1;
-ft->info.style = UNSIGNED;
-ft->info.size = BYTE;
+ft->info.encoding = ST_ENCODING_UNSIGNED;
+ft->info.size = ST_SIZE_BYTE;
 
 /* sounder header */
-wshort (ft,0); /* sample size code */
-wshort (ft,(int) ft->info.rate);     /* sample rate */
-wshort (ft,10);        /* volume */
-wshort (ft,4); /* shift */
+st_writew (ft,0); /* sample size code */
+st_writew (ft,(int) ft->info.rate);     /* sample rate */
+st_writew (ft,10);        /* volume */
+st_writew (ft,4); /* shift */
+
+return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         SNDTWRITE                                     */
 /*======================================================================*/
 
-void sndtwrite(ft, buf, len)
+LONG st_sndtwrite(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
 	struct sndpriv *p = (struct sndpriv *) ft->priv;
 	p->nsamples += len;
-	rawwrite(ft, buf, len);
+	return st_rawwrite(ft, buf, len);
 }
 
 /*======================================================================*/
 /*                         SNDTSTOPWRITE                                */
 /*======================================================================*/
 
-void sndtstopwrite(ft)
+int st_sndtstopwrite(ft)
 ft_t ft;
 {
 	struct sndpriv *p = (struct sndpriv *) ft->priv;
+	int rc;
 
 	/* Flush remaining buffer out */
-	rawstopwrite(ft);
+	rc = st_rawstopwrite(ft);
+	if (rc)
+	    return rc;
 
 	/* fixup file sizes in header */
 	if (fseek(ft->fp, 0L, 0) != 0)
-		fail("can't rewind output file to rewrite SND header");
+		st_fail("can't rewind output file to rewrite SND header");
 	sndtwriteheader(ft, p->nsamples);
+
+	return(ST_SUCCESS);
 }
 
 /*======================================================================*/
 /*                         SNDTWRITEHEADER                              */
 /*======================================================================*/
-void sndtwriteheader(ft,nsamples)
+static void sndtwriteheader(ft,nsamples)
 ft_t ft;
 LONG nsamples;
 {
 char name_buf[97];
 
 /* sndtool header */
-fputs ("SOUND",ft->fp); /* magic */
-fputc (0x1a,ft->fp);
-wshort (ft,(LONG)0);  /* hGSound */
-wlong (ft,nsamples);
-wlong (ft,(LONG)0);
-wlong (ft,nsamples);
-wshort (ft,(int) ft->info.rate);
-wshort (ft,0);
-wshort (ft,10);
-wshort (ft,4);
+st_writes(ft, "SOUND"); /* magic */
+st_writeb(ft, 0x1a);
+st_writew (ft,(LONG)0);  /* hGSound */
+st_writedw (ft,nsamples);
+st_writedw (ft,(LONG)0);
+st_writedw (ft,nsamples);
+st_writew (ft,(int) ft->info.rate);
+st_writew (ft,0);
+st_writew (ft,10);
+st_writew (ft,4);
 memset (name_buf, 0, 96);
 sprintf (name_buf,"%s - File created by Sound Exchange",ft->filename);
 fwrite (name_buf, 1, 96, ft->fp);
 }
 
-
