@@ -11,18 +11,18 @@ t() {
 t 8svx
 t aiff
 t au 
+t avr
 t cdr
 t cvs
 t dat
-t vms
 t hcom -r 22050
 t maud
 t raw -r 8130 -t ub
 t sf 
 t smp
-t sndr
 t sndt 
 t txw
+t vms
 t voc
-t wav 
-t wve 
+t wav
+t wve
