@@ -145,12 +145,12 @@ BLOCK 9 - data block that supersedes blocks 1 and 8.
 typedef struct vocstuff {
 	LONG	rest;			/* bytes remaining in current block */
 	LONG	rate;			/* rate code (byte) of this chunk */
-	int		silent;			/* sound or silence? */
+	int		silent;		/* sound or silence? */
 	LONG	srate;			/* rate code (byte) of silence */
 	LONG	blockseek;		/* start of current output block */
 	LONG	samples;		/* number of samples output */
-	int		size;			/* word length of data */
-	int		channels;		/* number of sound channels */
+	int		size;		/* word length of data */
+	unsigned char	channels;	/* number of sound channels */
 	int     extended;       /* Has an extended block been read? */
 } *vs_t;
 
@@ -167,61 +167,83 @@ typedef struct vocstuff {
 
 #define	min(a, b)	(((a) < (b)) ? (a) : (b))
 
-void getblock();
-void blockstart(P1(ft_t));
-void blockstop(P1(ft_t));
+static int getblock(P1(ft_t));
+static void blockstart(P1(ft_t));
+static void blockstop(P1(ft_t));
 
-void vocstartread(ft) 
+int st_vocstartread(ft) 
 ft_t ft;
 {
 	char header[20];
 	vs_t v = (vs_t) ft->priv;
-	int sbseek;
-	int littlendian = 1;
-	char *endptr;
+	unsigned short sbseek;
+	int rc;
 
-	endptr = (char *) &littlendian;
 	/* VOC is in Little Endian format.  Swap bytes read in on */
 	/* Big Endian mahcines.				          */
-	if (!*endptr)
+	if (ST_IS_BIGENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
 
 	if (! ft->seekable)
-		fail("VOC input file must be a file, not a pipe");
+	{
+		st_fail("VOC input file must be a file, not a pipe");
+		return(ST_EOF);
+	}
 	if (fread(header, 1, 20, ft->fp) != 20)
-		fail("unexpected EOF in VOC header");
+	{
+		st_fail("unexpected EOF in VOC header");
+		return(ST_EOF);
+	}
 	if (strncmp(header, "Creative Voice File\032", 19))
-		fail("VOC file header incorrect");
+	{
+		st_fail("VOC file header incorrect");
+		return(ST_EOF);
+	}
 
-	sbseek = rshort(ft);
+	st_readw(ft, &sbseek);
 	fseek(ft->fp, sbseek, 0);
 
 	v->rate = -1;
 	v->rest = 0;
 	v->extended = 0;
-	getblock(ft);
+	rc = getblock(ft);
+	if (rc)
+	    return rc;
 	if (v->rate == -1)
-		fail("Input .voc file had no sound!");
+	{
+		st_fail("Input .voc file had no sound!");
+		return(ST_EOF);
+	}
 
 	ft->info.size = v->size;
-	ft->info.style = UNSIGNED;
-	if (v->size == WORD)
-	    ft->info.style = SIGN2;
+	ft->info.encoding = ST_ENCODING_UNSIGNED;
+	if (v->size == ST_SIZE_WORD)
+	    ft->info.encoding = ST_ENCODING_SIGN2;
 	if (ft->info.channels == -1)
 		ft->info.channels = v->channels;
+
+	return(ST_SUCCESS);
 }
 
-LONG vocread(ft, buf, len) 
+LONG st_vocread(ft, buf, len) 
 ft_t ft;
 LONG *buf, len;
 {
 	vs_t v = (vs_t) ft->priv;
 	int done = 0;
+	int rc;
+	unsigned short us;
+	unsigned char uc;
 	
 	if (v->rest == 0)
-		getblock(ft);
+	{
+		rc = getblock(ft);
+		if (rc)
+		    return 0;
+	}
+
 	if (v->rest == 0)
 		return 0;
 
@@ -231,27 +253,26 @@ LONG *buf, len;
 			*buf++ = 0x80000000L;
 	} else {
 		for(;v->rest && (done < len); v->rest--, done++) {
-			LONG datum;
 			switch(v->size)
 			{
-			    case BYTE:
-				if ((datum = getc(ft->fp)) == EOF) {
-				    warn("VOC input: short file");
+			    case ST_SIZE_BYTE:
+				if (st_readb(ft, &uc) == ST_EOF) {
+				    st_warn("VOC input: short file");
 				    v->rest = 0;
 				    return done;
 				}
-				datum ^= 0x80;	/* convert to signed */
-				*buf++ = LEFT(datum, 24);
+				uc ^= 0x80;	/* convert to signed */
+				*buf++ = LEFT(uc, 24);
 				break;
-			    case WORD:
-				datum = rshort(ft);
+			    case ST_SIZE_WORD:
+				st_readw(ft, &us);
 				if (feof(ft->fp))
 				{
-				    warn("VOC input: short file");
+				    st_warn("VOC input: short file");
 				    v->rest = 0;
 				    return done;
 				}
-				*buf++ = LEFT(datum, 16);
+				*buf++ = LEFT(us, 16);
 				v->rest--; /* Processed 2 bytes so update */
 				break;
 			}	
@@ -261,9 +282,10 @@ LONG *buf, len;
 }
 
 /* nothing to do */
-void vocstopread(ft) 
+int st_vocstopread(ft) 
 ft_t ft;
 {
+    return(ST_SUCCESS);
 }
 
 /* When saving samples in VOC format the following outline is followed:
@@ -277,47 +299,50 @@ ft_t ft;
  * which will work with the oldest software (eg. an 8-bit mono sample
  * will be able to be played with a really old SB VOC player.)
  */
-void vocstartwrite(ft) 
+int st_vocstartwrite(ft) 
 ft_t ft;
 {
 	vs_t v = (vs_t) ft->priv;
-	int littlendian = 1;
-	char *endptr;
 
-	endptr = (char *) &littlendian;
 	/* VOC is in Little Endian format.  Swap whats read */
 	/* in on Big Endian machines.			    */
-	if (!*endptr)
+	if (ST_IS_BIGENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
 
 	if (! ft->seekable)
-		fail("Output .voc file must be a file, not a pipe");
+	{
+		st_fail("Output .voc file must be a file, not a pipe");
+		return(ST_EOF);
+	}
 
 	v->samples = 0;
 
 	/* File format name and a ^Z (aborts printing under DOS) */
-	(void) fwrite("Creative Voice File\032\032", 1, 20, ft->fp);
-	wshort(ft, 26);			/* size of header */
-	wshort(ft, 0x10a);              /* major/minor version number */
-	wshort(ft, 0x1129);		/* checksum of version number */
+	st_writes(ft, "Creative Voice File\032");
+	st_writew(ft, 26);			/* size of header */
+	st_writew(ft, 0x10a);              /* major/minor version number */
+	st_writew(ft, 0x1129);		/* checksum of version number */
 
-	if (ft->info.size == BYTE)
-	  ft->info.style = UNSIGNED;
+	if (ft->info.size == ST_SIZE_BYTE)
+	  ft->info.encoding = ST_ENCODING_UNSIGNED;
 	else
-	  ft->info.style = SIGN2;
+	  ft->info.encoding = ST_ENCODING_SIGN2;
 	if (ft->info.channels == -1)
 		ft->info.channels = 1;
+
+	return(ST_SUCCESS);
 }
 
-void vocwrite(ft, buf, len) 
+LONG st_vocwrite(ft, buf, len) 
 ft_t ft;
 LONG *buf, len;
 {
 	vs_t v = (vs_t) ft->priv;
 	unsigned char uc;
 	int sw;
+	LONG done = 0;
 	
 	if (v->samples == 0) {
 	  /* No silence packing yet. */
@@ -325,116 +350,141 @@ LONG *buf, len;
 	  blockstart(ft);
 	}
 	v->samples += len;
-	while(len--) {
-	  if (ft->info.size == BYTE) {
+	while(done < len) {
+	  if (ft->info.size == ST_SIZE_BYTE) {
 	    uc = RIGHT(*buf++, 24);
 	    uc ^= 0x80;
-	    putc(uc, ft->fp);
+	    st_writeb(ft, uc);
 	  } else {
 		sw = (int) RIGHT(*buf++, 16);
-	    wshort(ft,sw);
+	    st_writew(ft,sw);
           }
+	  done++;
 	}
+	return done;
 }
 
-void vocstopwrite(ft) 
+int st_vocstopwrite(ft) 
 ft_t ft;
 {
 	blockstop(ft);
+	return(ST_SUCCESS);
 }
 
 /* Voc-file handlers */
 
 /* Read next block header, save info, leave position at start of data */
-void
+static int
 getblock(ft)
 ft_t ft;
 {
 	vs_t v = (vs_t) ft->priv;
 	unsigned char uc, block;
 	ULONG sblen;
-	LONG new_rate;
+	unsigned short new_rate_short;
+	ULONG new_rate_long;
 	int i;
+	ULONG trash;
 
 	v->silent = 0;
 	while (v->rest == 0) {
 		if (feof(ft->fp))
-			return;
-		block = getc(ft->fp);
+			return ST_SUCCESS;
+		st_readb(ft, &block);
 		if (block == VOC_TERM)
-			return;
+			return ST_SUCCESS;
 		if (feof(ft->fp))
-			return;
+			return ST_SUCCESS;
 		/* 
 		 * Size is an 24-bit value.  Currently there is no util 
 		 * func to read this so do it this cross-platform way
 		 *
 		 */
-		uc = getc(ft->fp);
+		st_readb(ft, &uc);
 		sblen = uc;
-		uc = getc(ft->fp);
+		st_readb(ft, &uc);
 		sblen |= ((LONG) uc) << 8;
-		uc = getc(ft->fp);
+		st_readb(ft, &uc);
 		sblen |= ((LONG) uc) << 16;
 		switch(block) {
 		case VOC_DATA: 
-		        uc = getc(ft->fp);
+			st_readb(ft, &uc);
 			/* When DATA block preceeded by an EXTENDED     */
 			/* block, the DATA blocks rate value is invalid */
 		        if (!v->extended) {
 			  if (uc == 0)
-			    fail("File %s: Sample rate is zero?");
+			  {
+			    st_fail("File %s: Sample rate is zero?");
+			    return(ST_EOF);
+			  }
 			  if ((v->rate != -1) && (uc != v->rate))
-			    fail("File %s: sample rate codes differ: %d != %d",
+			  {
+			    st_fail("File %s: sample rate codes differ: %d != %d",
 				 ft->filename,v->rate, uc);
+			    return(ST_EOF);
+			  }
 			  v->rate = uc;
 			  ft->info.rate = 1000000.0/(256 - v->rate);
 			  v->channels = 1;
 			}
-			uc = getc(ft->fp);
+			st_readb(ft, &uc);
 			if (uc != 0)
-			  fail("File %s: only interpret 8-bit data!",
+			{
+			  st_fail("File %s: only interpret 8-bit data!",
 			       ft->filename);
+			  return(ST_EOF);
+			}
 			v->extended = 0;
 			v->rest = sblen - 2;
-			v->size = BYTE;
-			return;
+			v->size = ST_SIZE_BYTE;
+			return (ST_SUCCESS);
 		case VOC_DATA_16:
-			new_rate = rlong(ft);
-			if (new_rate == 0)
-			    fail("File %s: Sample rate is zero?",ft->filename);
-			if ((v->rate != -1) && (new_rate != v->rate))
-			    fail("File %s: sample rate codes differ: %d != %d",
-				ft->filename, v->rate, new_rate);
-			v->rate = new_rate;
-			ft->info.rate = new_rate;
-			uc = getc(ft->fp);
+			st_readdw(ft, &new_rate_long);
+			if (new_rate_long == 0)
+			{
+			    st_fail("File %s: Sample rate is zero?",ft->filename);
+			    return(ST_EOF);
+			}
+			if ((v->rate != -1) && (new_rate_long != v->rate))
+			{
+			    st_fail("File %s: sample rate codes differ: %d != %d",
+				ft->filename, v->rate, new_rate_long);
+			    return(ST_EOF);
+			}
+			v->rate = new_rate_long;
+			ft->info.rate = new_rate_long;
+			st_readb(ft, &uc);
 			switch (uc)
 			{
-			    case 8:	v->size = BYTE; break;
-			    case 16:	v->size = WORD; break;
-			    default:	fail("Don't understand size %d", uc);
+			    case 8:	v->size = ST_SIZE_BYTE; break;
+			    case 16:	v->size = ST_SIZE_WORD; break;
+			    default:	
+					st_fail("Don't understand size %d", uc);
+					return(ST_EOF);
 			}
-			v->channels = getc(ft->fp);
-			getc(ft->fp);	/* unknown */
-			getc(ft->fp);	/* notused */
-			getc(ft->fp);	/* notused */
-			getc(ft->fp);	/* notused */
-			getc(ft->fp);	/* notused */
-			getc(ft->fp);	/* notused */
+			st_readb(ft, &(v->channels));
+			st_readb(ft, (unsigned char *)&trash); /* unknown */
+			st_readb(ft, (unsigned char *)&trash); /* notused */
+			st_readb(ft, (unsigned char *)&trash); /* notused */
+			st_readb(ft, (unsigned char *)&trash); /* notused */
+			st_readb(ft, (unsigned char *)&trash); /* notused */
+			st_readb(ft, (unsigned char *)&trash); /* notused */
 			v->rest = sblen - 12;
-			return;
+			return (ST_SUCCESS);
 		case VOC_CONT: 
 			v->rest = sblen;
-			return;
+			return (ST_SUCCESS);
 		case VOC_SILENCE: 
 			{
 			unsigned short period;
 
-			period = rshort(ft);
-			uc = getc(ft->fp);
+			st_readw(ft, &period);
+			st_readb(ft, &uc);
 			if (uc == 0)
-				fail("File %s: Silence sample rate is zero");
+			{
+				st_fail("File %s: Silence sample rate is zero");
+				return(ST_EOF);
+			}
 			/* 
 			 * Some silence-packed files have gratuitously
 			 * different sample rate codes in silence.
@@ -446,25 +496,25 @@ ft_t ft;
 				v->rate = uc;
 			v->rest = period;
 			v->silent = 1;
-			return;
+			return (ST_SUCCESS);
 			}
 		case VOC_MARKER:
-			uc = getc(ft->fp);
-			uc = getc(ft->fp);
+			st_readb(ft, &uc);
+			st_readb(ft, &uc);
 			/* Falling! Falling! */
 		case VOC_TEXT:
 			{
 			int i;
 			/* Could add to comment in SF? */
 			for(i = 0; i < sblen; i++)
-				getc(ft->fp);
+			    st_readb(ft, (unsigned char *)&trash);
 			}
 			continue;	/* get next block */
 		case VOC_LOOP:
 		case VOC_LOOPEND:
-			report("File %s: skipping repeat loop");
+			st_report("File %s: skipping repeat loop");
 			for(i = 0; i < sblen; i++)
-				getc(ft->fp);
+			    st_readb(ft, (unsigned char *)&trash);
 			break;
 		case VOC_EXTENDED:
 			/* An Extended block is followed by a data block */
@@ -472,18 +522,27 @@ ft_t ft;
 			/* value from the extended block and not the     */
 			/* data block.					 */
 			v->extended = 1;
-			new_rate = rshort(ft);
-			if (new_rate == 0)
-			   fail("File %s: Sample rate is zero?");
-			if ((v->rate != -1) && (new_rate != v->rate))
-			   fail("File %s: sample rate codes differ: %d != %d",
-					ft->filename, v->rate, new_rate);
-			v->rate = new_rate;
-			uc = getc(ft->fp);
+			st_readw(ft, &new_rate_short);
+			if (new_rate_short == 0)
+			{
+			   st_fail("File %s: Sample rate is zero?");
+			   return(ST_EOF);
+			}
+			if ((v->rate != -1) && (new_rate_short != v->rate))
+			{
+			   st_fail("File %s: sample rate codes differ: %d != %d",
+					ft->filename, v->rate, new_rate_short);
+			   return(ST_EOF);
+			}
+			v->rate = new_rate_short;
+			st_readb(ft, &uc);
 			if (uc != 0)
-				fail("File %s: only interpret 8-bit data!",
+			{
+				st_fail("File %s: only interpret 8-bit data!",
 					ft->filename);
-			uc = getc(ft->fp);
+				return(ST_EOF);
+			}
+			st_readb(ft, &uc);
 			if (uc)
 				ft->info.channels = 2;  /* Stereo */
 			/* Needed number of channels before finishing
@@ -494,28 +553,29 @@ ft_t ft;
 			/* can be grabed.				*/
 			continue;
 		default:
-			report("File %s: skipping unknown block code %d",
+			st_report("File %s: skipping unknown block code %d",
 				ft->filename, block);
 			for(i = 0; i < sblen; i++)
-				getc(ft->fp);
+			    st_readb(ft, (unsigned char *)&trash);
 		}
 	}
+	return ST_SUCCESS;
 }
 
 /* Start an output block. */
-void blockstart(ft)
+static void blockstart(ft)
 ft_t ft;
 {
 	vs_t v = (vs_t) ft->priv;
 
 	v->blockseek = ftell(ft->fp);
 	if (v->silent) {
-		putc(VOC_SILENCE, ft->fp);	/* Silence block code */
-		putc(0, ft->fp);		/* Period length */
-		putc(0, ft->fp);		/* Period length */
-		putc((int) v->rate, ft->fp);		/* Rate code */
+		st_writeb(ft, VOC_SILENCE);	/* Silence block code */
+		st_writeb(ft, 0);		/* Period length */
+		st_writeb(ft, 0);		/* Period length */
+		st_writeb(ft, v->rate);		/* Rate code */
 	} else {
-	  if (ft->info.size == BYTE) {
+	  if (ft->info.size == ST_SIZE_BYTE) {
 	    /* 8-bit sample section.  By always setting the correct     */
 	    /* rate value in the DATA block (even when its preceeded    */
 	    /* by an EXTENDED block) old software can still play stereo */
@@ -523,66 +583,66 @@ ft_t ft;
 	    /* Prehaps the rate should be doubled though to make up for */
 	    /* double amount of samples for a given time????            */
 	    if (ft->info.channels > 1) {
-	      putc(VOC_EXTENDED, ft->fp);	/* Voice Extended block code */
-	      putc(4, ft->fp);                /* block length = 4 */
-	      putc(0, ft->fp);                /* block length = 4 */
-	      putc(0, ft->fp);                /* block length = 4 */
+	      st_writeb(ft, VOC_EXTENDED);	/* Voice Extended block code */
+	      st_writeb(ft, 4);                /* block length = 4 */
+	      st_writeb(ft, 0);                /* block length = 4 */
+	      st_writeb(ft, 0);                /* block length = 4 */
 		  v->rate = 65536L - (256000000.0/(2*(float)ft->info.rate));
-	      wshort(ft,v->rate);	/* Rate code */
-	      putc(0, ft->fp);                /* File is not packed */
-	      putc(1, ft->fp);                /* samples are in stereo */
+	      st_writew(ft,v->rate);	/* Rate code */
+	      st_writeb(ft, 0);         /* File is not packed */
+	      st_writeb(ft, 1);         /* samples are in stereo */
 	    }
-	    putc(VOC_DATA, ft->fp);		/* Voice Data block code */
-	    putc(0, ft->fp);		/* block length (for now) */
-	    putc(0, ft->fp);		/* block length (for now) */
-	    putc(0, ft->fp);		/* block length (for now) */
+	    st_writeb(ft, VOC_DATA);	/* Voice Data block code */
+	    st_writeb(ft, 0);		/* block length (for now) */
+	    st_writeb(ft, 0);		/* block length (for now) */
+	    st_writeb(ft, 0);		/* block length (for now) */
 	    v->rate = 256 - (1000000.0/(float)ft->info.rate);
-	    putc((int) v->rate, ft->fp);/* Rate code */
-	    putc(0, ft->fp);		/* 8-bit raw data */
+	    st_writeb(ft, (int) v->rate);/* Rate code */
+	    st_writeb(ft, 0);		/* 8-bit raw data */
 	} else {
-	    putc(VOC_DATA_16, ft->fp);		/* Voice Data block code */
-	    putc(0, ft->fp);		/* block length (for now) */
-	    putc(0, ft->fp);		/* block length (for now) */
-	    putc(0, ft->fp);		/* block length (for now) */
+	    st_writeb(ft, VOC_DATA_16); /* Voice Data block code */
+	    st_writeb(ft, 0);		/* block length (for now) */
+	    st_writeb(ft, 0);		/* block length (for now) */
+	    st_writeb(ft, 0);		/* block length (for now) */
 	    v->rate = ft->info.rate;
-	    wlong(ft, v->rate);	/* Rate code */
-	    putc(16, ft->fp);		/* Sample Size */
-	    putc(ft->info.channels, ft->fp);	/* Sample Size */
-	    putc(0, ft->fp);		/* Unknown */
-	    putc(0, ft->fp);		/* Unused */
-	    putc(0, ft->fp);		/* Unused */
-	    putc(0, ft->fp);		/* Unused */
-	    putc(0, ft->fp);		/* Unused */
-	    putc(0, ft->fp);		/* Unused */
+	    st_writedw(ft, v->rate);	/* Rate code */
+	    st_writeb(ft, 16);		/* Sample Size */
+	    st_writeb(ft, ft->info.channels);	/* Sample Size */
+	    st_writeb(ft, 0);		/* Unknown */
+	    st_writeb(ft, 0);		/* Unused */
+	    st_writeb(ft, 0);		/* Unused */
+	    st_writeb(ft, 0);		/* Unused */
+	    st_writeb(ft, 0);		/* Unused */
+	    st_writeb(ft, 0);		/* Unused */
 	  }
 	}
 }
 
 /* End the current data or silence block. */
-void blockstop(ft) 
+static void blockstop(ft) 
 ft_t ft;
 {
 	vs_t v = (vs_t) ft->priv;
 	LONG datum;
 
-	putc(0, ft->fp);			/* End of file block code */
+	st_writeb(ft, 0);			/* End of file block code */
 	fseek(ft->fp, v->blockseek, 0);		/* seek back to block length */
 	fseek(ft->fp, 1, 1);			/* seek forward one */
 	if (v->silent) {
-		wshort(ft, v->samples);
+		st_writew(ft, v->samples);
 	} else {
-	  if (ft->info.size == BYTE) {
+	  if (ft->info.size == ST_SIZE_BYTE) {
 	    if (ft->info.channels > 1) {
 	      fseek(ft->fp, 8, 1); /* forward 7 + 1 for new block header */
 	    }
 	  }
 	        v->samples += 2;		/* adjustment: SBDK pp. 3-5 */
 		datum = (v->samples) & 0xff;
-		putc((int)datum, ft->fp);       /* low byte of length */
+		st_writeb(ft, (int)datum);       /* low byte of length */
 		datum = (v->samples >> 8) & 0xff;
-		putc((int)datum, ft->fp);  /* middle byte of length */
+		st_writeb(ft, (int)datum);  /* middle byte of length */
 		datum = (v->samples >> 16) & 0xff;
-		putc((int)datum, ft->fp); /* high byte of length */
+		st_writeb(ft, (int)datum); /* high byte of length */
 	}
 }
 
