@@ -5,6 +5,7 @@
 
 #include "st.h"
 #include "g72x.h"
+#include "string.h"
 
 /* Magic numbers used in Psion audio files */
 #define PSION_MAGIC     "ALawSoundFile**"
@@ -14,52 +15,48 @@
 
 struct wvepriv
     {
-    unsigned int length;
+    ULONG length;
     short padding;
     short repeats;
     };
 
-void wvewriteheader(P1(ft_t ft));
-LONG rawread(P3(ft_t, LONG *, LONG));
-void rawwrite(P3(ft_t, LONG *, LONG));
+static void wvewriteheader(P1(ft_t ft));
 
-void wvestartread(ft) 
+int st_wvestartread(ft) 
 ft_t ft;
 {
 	struct wvepriv *p = (struct wvepriv *) ft->priv;
 	char magic[16];
 	short version;
+	int rc;
 
-	int littlendian = 1;
-	char *endptr;
+	ULONG trash;
 
 	/* Needed for rawread() */
-	rawstartread(ft);
+	rc = st_rawstartread(ft);
+	if (rc)
+	    return rc;
 
-	endptr = (char *) &littlendian;
 	/* WVE is in big endian format.  Swap whats read in
 	 * on little endian machines.
 	 */
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
 
-	/* Sanity check */ 
-	if (sizeof(struct wvepriv) > PRIVSIZE)
-		fail(
-"struct wvepriv is too big (%d); change PRIVSIZE in st.h and recompile sox",
-		     sizeof(struct wvepriv));
-
-	/* Check the magic word */
-        fread(magic, 16, 1, ft->fp);
-	if (strcmp(magic, PSION_MAGIC)==0) {
-		report("Found Psion magic word");
+	/* Check the magic word (null-terminated) */
+        st_reads(ft, magic, 16);
+	if (strncmp(magic, PSION_MAGIC, 15)==0) {
+		st_report("Found Psion magic word");
 	}
 	else
-		fail("Psion header doesn't start with magic word\nTry the '.al' file type with '-t al -r 8000 filename'");
+	{
+		st_fail("Psion header doesn't start with magic word\nTry the '.al' file type with '-t al -r 8000 filename'");
+		return (ST_EOF);
+	}
 
-        version=rshort(ft);
+        st_readw(ft, &version);
 
 	/* Check for what type endian machine its read on */
 	if (version == PSION_INV_VERSION)
@@ -69,31 +66,35 @@ ft_t ft;
 		 * it doesn't hurt.
 		 */
 		ft->swap = ft->swap ? 0 : 1;
-		report("Found inverted PSION magic word.  Swapping bytes.");
+		st_report("Found inverted PSION magic word.  Swapping bytes.");
 	}
 	else if (version == PSION_VERSION)
 	{
-	    report("Found PSION magic word");
+	    st_report("Found PSION magic word");
 	}
 	else
-	    fail("Wrong version in Psion header");
+	{
+	    st_fail("Wrong version in Psion header");
+	    return(ST_EOF);
+	}
 
-     	p->length=rlong(ft);
+     	st_readdw(ft, &(p->length));
 
-	p->padding=rshort(ft);
+	st_readw(ft, &(p->padding));
 
-	p->repeats=rshort(ft);
+	st_readw(ft, &(p->repeats));
 
- 	(void)rshort(ft);
- 	(void)rshort(ft);
- 	(void)rshort(ft);
+ 	(void)st_readw(ft, (unsigned short *)&trash);
+  	(void)st_readw(ft, (unsigned short *)&trash);
+	(void)st_readw(ft, (unsigned short *)&trash);
     
-	ft->info.style = ALAW;
-	ft->info.size = BYTE;
+	ft->info.encoding = ST_ENCODING_ALAW;
+	ft->info.size = ST_SIZE_BYTE;
 
 	ft->info.rate = 8000;
 
 	ft->info.channels = 1;
+	return (ST_SUCCESS);
 }
 
 /* When writing, the header is supposed to contain the number of
@@ -105,22 +106,21 @@ ft_t ft;
    if it is not, the unspecified size remains in the header
    (this is illegal). */
 
-void wvestartwrite(ft) 
+int st_wvestartwrite(ft) 
 ft_t ft;
 {
 	struct wvepriv *p = (struct wvepriv *) ft->priv;
-
-	int littlendian = 1;
-	char *endptr;
+	int rc;
 
 	/* Needed for rawwrite() */
-	rawstartwrite(ft);
+	rc = st_rawstartwrite(ft);
+	if (rc)
+	    return ST_EOF;
 
-	endptr = (char *) &littlendian;
 	/* wve is in big endian format.  Swap whats read in
 	 * on little endian machines.
 	 */
-	if (*endptr)
+	if (ST_IS_LITTLEENDIAN)
 	{
 		ft->swap = ft->swap ? 0 : 1;
 	}
@@ -129,45 +129,51 @@ ft_t ft;
 	if (p->repeats == 0)
 	    p->repeats = 1;
 
-	ft->info.style = ALAW;
-	ft->info.size = BYTE;
+	ft->info.encoding = ST_ENCODING_ALAW;
+	ft->info.size = ST_SIZE_BYTE;
 	ft->info.rate = 8000;
 
 	wvewriteheader(ft);
+	return ST_SUCCESS;
 }
 
-LONG wveread(ft, buf, samp)
+LONG st_wveread(ft, buf, samp)
 ft_t ft;
 LONG *buf, samp;
 {
-	return rawread(ft, buf, samp);
+	return st_rawread(ft, buf, samp);
 }
 
-void wvewrite(ft, buf, samp)
+LONG st_wvewrite(ft, buf, samp)
 ft_t ft;
 LONG *buf, samp;
 {
 	struct wvepriv *p = (struct wvepriv *) ft->priv;
 	p->length += samp * ft->info.size;
-	rawwrite(ft, buf, samp);
+	return st_rawwrite(ft, buf, samp);
 }
 
-void
-wvestopwrite(ft)
+int st_wvestopwrite(ft)
 ft_t ft;
 {
 	if (!ft->seekable)
-		return;
+	{
+	    st_warn("Header will be have invalid file length since file is not seekable");
+	    return ST_SUCCESS;
+	}
 
 	if (fseek(ft->fp, 0L, 0) != 0)
-		fail("Can't rewind output file to rewrite Psion header.");
+	{
+		st_fail("Can't rewind output file to rewrite Psion header.");
+		return(ST_EOF);
+	}
 	wvewriteheader(ft);
 
 	/* Needed for rawwrite() */
-	rawstopwrite(ft);
+	return st_rawstopwrite(ft);
 }
 
-void wvewriteheader(ft)
+static void wvewriteheader(ft)
 ft_t ft;
 {
 
@@ -180,15 +186,17 @@ ft_t ft;
     version=PSION_VERSION;
     zero=0;
 
-    fwrite(magic, sizeof(magic), 1, ft->fp);
+    st_writes(ft, magic);
+    /* Null terminate string */
+    st_writeb(ft, 0);
 
-    wshort(ft, version);
-    wlong(ft, p->length);
-    wshort(ft, p->padding);
-    wshort(ft, p->repeats);
+    st_writew(ft, version);
+    st_writedw(ft, p->length);
+    st_writew(ft, p->padding);
+    st_writew(ft, p->repeats);
 
-    wshort(ft, zero);
-    wshort(ft, zero);
-    wshort(ft, zero);
+    st_writew(ft, zero);
+    st_writew(ft, zero);
+    st_writew(ft, zero);
 }
 
